@@ -182,8 +182,6 @@ static comedi_lrange range_pci171x_da={ 2, {
 static int pci1710_attach(comedi_device *dev,comedi_devconfig *it);
 static int pci1710_detach(comedi_device *dev);
 
-static unsigned short	pci_list_builded=0;	/*=1 list of card is know */
-
 typedef struct {
 	char 		*name;		// driver name
 	int		device_id;
@@ -229,13 +227,13 @@ static boardtype boardtypes[] =
 	 10000, 2048 },
 	{"pci1711", 0x1711,
 	 IORANGE_171x, 1, TYPE_PCI171X,
-	 16, 0, 2, 16, 16, 0,  0x0fff, 0x0fff,
+	 16, 0, 2, 16, 16, 1,  0x0fff, 0x0fff,
 	 &range_pci17x1, range_codes_pci17x1, &range_pci171x_da,
 	 10000, 512 },
 	{"pci1713", 0x1713,
 	 IORANGE_171x, 1, TYPE_PCI1713,
 	 32,16, 0,  0,  0, 0,  0x0fff, 0x0000,
-	 &range_pci17x1, range_codes_pci1710_3, NULL,
+	 &range_pci1710_3, range_codes_pci1710_3, NULL,
 	 10000, 2048 },
 	{"pci1720", 0x1720,
 	 IORANGE_1720, 0, TYPE_PCI1720,
@@ -262,6 +260,7 @@ static comedi_driver driver_pci1710={
 };
 
 typedef struct{
+	struct pci_dev		*pcidev;	// ptr to PCI device
 	char			valid;		// card is usable
 	char			neverending_ai;	// we do unlimited AI
 	unsigned int		CntrlReg;	// Control register
@@ -457,7 +456,7 @@ static int pci171x_insn_counter_read(comedi_device *dev,comedi_subdevice *s, com
 
 		data[0] = lsb | (msb << 8);
 	}
-#if unused
+#if 0
 	if ( /* count and status */ )
 		ccntrl = 0xC2;
 		/* status, if latched, first */
@@ -719,7 +718,6 @@ static void interrupt_pci1710_half_fifo(void *d)
 static irqreturn_t interrupt_service_pci1710(int irq, void *d, struct pt_regs *regs)
 {
 	comedi_device *dev = d;
-	int retval = 1;
 
 	DPRINTK("adv_pci1710 EDBG: BGN: interrupt_service_pci1710(%d,...)\n",irq);
 	if (!(inw(dev->iobase + PCI171x_STATUS) & Status_IRQ)) 	// is this interrupt from our board?
@@ -739,7 +737,7 @@ static irqreturn_t interrupt_service_pci1710(int irq, void *d, struct pt_regs *r
 		outw(devpriv->CntrlReg, dev->iobase+PCI171x_CONTROL);
 		// start pacer
 		start_pacer(dev, 1, devpriv->ai_et_div1, devpriv->ai_et_div2);
-		return IRQ_RETVAL(retval);
+		return IRQ_HANDLED;
 	}
 	if (devpriv->ai_eos) {  // We use FIFO half full INT or not?
 		interrupt_pci1710_every_sample(d);
@@ -747,7 +745,7 @@ static irqreturn_t interrupt_service_pci1710(int irq, void *d, struct pt_regs *r
 		interrupt_pci1710_half_fifo(d);
 	}
 	DPRINTK("adv_pci1710 EDBG: END: interrupt_service_pci1710(...)\n");
-	return IRQ_RETVAL(retval);
+	return IRQ_HANDLED;
 }
 
 /*
@@ -1157,7 +1155,7 @@ static int pci171x_reset(comedi_device *dev)
 {
 	DPRINTK("adv_pci1710 EDBG: BGN: pci171x_reset(...)\n");
 	outw(0x30, dev->iobase + PCI171x_CNTCTRL);
-	devpriv->CntrlReg=Control_SW;	// Software trigger, CNT0=100kHz
+	devpriv->CntrlReg=Control_SW | Control_CNT0;	// Software trigger, CNT0=external
 	outw(devpriv->CntrlReg, dev->iobase+PCI171x_CONTROL);	// reset any operations
 	outb(0, dev->iobase + PCI171x_CLRFIFO);		// clear FIFO
 	outb(0, dev->iobase + PCI171x_CLRINT);		// clear INT request
@@ -1222,55 +1220,72 @@ static int pci1710_attach(comedi_device *dev,comedi_devconfig *it)
 {
 	comedi_subdevice *s;
 	int ret,subdev,n_subdevices;
-	unsigned short io_addr[5],master,irq;
-	struct pcilst_struct *card=NULL;
+	unsigned short irq;
         unsigned int iobase;
+	struct pci_dev *pcidev;
+	int opt_bus, opt_slot;
+	const char *errstr;
 	unsigned char pci_bus,pci_slot,pci_func;
 
-	if (!pci_list_builded) {
-		pci_card_list_init(PCI_VENDOR_ID_ADVANTECH,
-#ifdef PCI171X_EXTDEBUG
-						    1
-#else
-						    0
-#endif
-		);
-		pci_list_builded=1;
-	}
-
 	rt_printk("comedi%d: adv_pci1710: board=%s",dev->minor,this_board->name);
 
-	/* this call pci_enable_device() */
-	if ((card=select_and_alloc_pci_card(PCI_VENDOR_ID_ADVANTECH, this_board->device_id, it->options[0], it->options[1], 0))==NULL)
-		return -EIO;
+	opt_bus = it->options[0];
+	opt_slot = it->options[1];
 
-	if ((pci_card_data(card,&pci_bus,&pci_slot,&pci_func,
-	                    &io_addr[0],&irq,&master))<0) {
-		pci_card_free(card);
-		rt_printk(" - Can't get configuration data!\n");
+	if((ret=alloc_private(dev,sizeof(pci1710_private)))<0) {
+		rt_printk(" - Allocation failed!\n");
+		return -ENOMEM;
+	}
+
+	/* Look for matching PCI device */
+	errstr = "not found!";
+	pcidev = NULL;
+	while (NULL != (pcidev = pci_get_device(PCI_VENDOR_ID_ADVANTECH,
+					this_board->device_id, pcidev))) {
+		/* Found matching vendor/device. */
+		if (opt_bus || opt_slot) {
+			/* Check bus/slot. */
+			if (opt_bus != pcidev->bus->number
+					|| opt_slot != PCI_SLOT(pcidev->devfn))
+				continue;	/* no match */
+		}
+		/*
+		 * Look for device that isn't in use.
+		 * Enable PCI device and request regions.
+		 */
+		if (pci_enable_device(pcidev)) {
+			errstr = "failed to enable PCI device!";
+			continue;
+		}
+		if (pci_request_regions(pcidev, "adl_pci9118")) {
+			errstr = "in use or I/O port conflict!";
+			continue;
+		}
+		break;
+	}
+
+	if (!pcidev) {
+		if (opt_bus || opt_slot) {
+			rt_printk(" - Card at b:s %d:%d %s\n",
+					opt_bus, opt_slot, errstr);
+		} else {
+			rt_printk(" - Card %s\n", errstr);
+		}
 		return -EIO;
 	}
 
-	iobase=io_addr[2];
+	pci_bus = pcidev->bus->number;
+	pci_slot = PCI_SLOT(pcidev->devfn);
+	pci_func = PCI_FUNC(pcidev->devfn);
+	irq = pcidev->irq;
+	iobase = pci_resource_start(pcidev, 2);
 
 	rt_printk(", b:s:f=%d:%d:%d, io=0x%4x",pci_bus,pci_slot,pci_func,iobase);
 
-        if (check_region(iobase, this_board->iorange) < 0) {
-		pci_card_free(card);
-		rt_printk("I/O port conflict\n");
-		return -EIO;
-        }
-
-        request_region(iobase, this_board->iorange, "Advantech PCI-1710");
         dev->iobase=iobase;
 
 	dev->board_name = this_board->name;
-
-	if((ret=alloc_private(dev,sizeof(pci1710_private)))<0) {
-    		release_region(dev->iobase, this_board->iorange);
-		pci_card_free(card);
-		return -ENOMEM;
-	}
+	devpriv->pcidev = pcidev;
 
         n_subdevices = 0;
 	if (this_board->n_aichan) n_subdevices++;
@@ -1280,8 +1295,7 @@ static int pci1710_attach(comedi_device *dev,comedi_devconfig *it)
 	if (this_board->n_counter) n_subdevices++;
 
         if((ret=alloc_subdevices(dev, n_subdevices))<0) {
-    		release_region(dev->iobase, this_board->iorange);
-		pci_card_free(card);
+		rt_printk(" - Allocation failed!\n");
     		return ret;
 	}
 
@@ -1400,15 +1414,16 @@ static int pci1710_attach(comedi_device *dev,comedi_devconfig *it)
 static int pci1710_detach(comedi_device *dev)
 {
 	
-	if (dev->private) 
+	if (dev->private) {
 		if (devpriv->valid) pci1710_reset(dev);
-	
-	if (dev->irq) comedi_free_irq(dev->irq,dev);
-	if (dev->iobase) release_region(dev->iobase,this_board->iorange);
-
-	if (pci_list_builded) {
-	    	pci_card_list_cleanup(PCI_VENDOR_ID_ADVANTECH);
-		pci_list_builded=0;
+		if (dev->irq) comedi_free_irq(dev->irq,dev);
+		if (devpriv->pcidev) {
+			if (dev->iobase) {
+				pci_release_regions(devpriv->pcidev);
+				pci_disable_device(devpriv->pcidev);
+			}
+			pci_dev_put(devpriv->pcidev);
+		}
 	}
 
 
