@@ -295,7 +295,7 @@ static boardtype boardtypes[] =
 	  { 4, PCI175x_BOARDID, 1, SDF_INTERNAL},
 	 IO_16b
 	},
-	{"pci1760", PCI_VENDOR_ID_ADVANTECH, 0x1760, PCIDIO_MAINREG,
+	{"pci1760", PCI_VENDOR_ID_ADVANTECH, 0x1760, 0,
 	 TYPE_PCI1760,
 	 {{ 0, 0, 0, 0}, { 0, 0, 0, 0}},	// This card have own setup work
 	 {{ 0, 0, 0, 0}, { 0, 0, 0, 0}},
@@ -330,7 +330,6 @@ struct pci_dio_private_st {
 	pci_dio_private		*next;		// next private struct
 	struct pci_dev 		*pcidev;	// pointer to board's pci_dev
 	char			valid;		// card is usable
-	char			enabled;	// PCI card is enabled
 	char			GlobalIrqEnabled;// 1= any IRQ source is enabled
 	// PCI-1760 specific data
 	unsigned char		IDICntEnable;	// counter's counting enable status
@@ -428,7 +427,7 @@ static int pci_dio_insn_bits_do_w(comedi_device *dev, comedi_subdevice *s,
 /*
 ==============================================================================
 */
-static int pci1760_mbxrequest(comedi_device *dev, comedi_subdevice *s,
+static int pci1760_unchecked_mbxrequest(comedi_device *dev,
 	unsigned char *omb, unsigned char *imb, 
 	int repeats)
 {
@@ -456,51 +455,43 @@ static int pci1760_mbxrequest(comedi_device *dev, comedi_subdevice *s,
 	return -ETIME;
 }
 
+static int pci1760_clear_imb2(comedi_device *dev)
+{
+	unsigned char omb[4] = {0x0, 0x0, CMD_ClearIMB2, 0x0};
+	unsigned char imb[4];
+	/* check if imb2 is already clear */
+	if(inb(dev->iobase + IMB2) == CMD_ClearIMB2) return 0;
+	return pci1760_unchecked_mbxrequest(dev, omb, imb, OMBCMD_RETRY);
+}
+
+static int pci1760_mbxrequest(comedi_device *dev,
+	unsigned char *omb, unsigned char *imb)
+{
+	if(omb[2] == CMD_ClearIMB2)
+	{
+		comedi_error(dev, "bug! this function should not be used for CMD_ClearIMB2 command");
+		return -EINVAL;
+	}
+	if(inb(dev->iobase + IMB2) == omb[2])
+	{
+		int retval;
+		retval = pci1760_clear_imb2(dev);
+		if(retval < 0) return retval;
+	}
+	return pci1760_unchecked_mbxrequest(dev, omb, imb, OMBCMD_RETRY);
+}
+
 /*
 ==============================================================================
 */
 static int pci1760_insn_bits_di(comedi_device *dev, comedi_subdevice *s, 
 	comedi_insn *insn, lsampl_t *data)
 {
-	int ret;
-	unsigned char omb[4]={
-		0x00, 
-		0x00, 
-		CMD_ClearIMB2, 
-		0x00};
-	unsigned char imb[4];
-	
-	if (!(ret=pci1760_mbxrequest(dev, s, omb, imb, OMBCMD_RETRY)))
-		return ret;
-
-	data[1]=imb[3];
+	data[1]=inb(dev->iobase + IMB3);
 
 	return 2;
 }
 
-/*
-==============================================================================
-*/
-static int pci1760_insn_read_di(comedi_device *dev, comedi_subdevice *s, 
-	comedi_insn *insn, lsampl_t *data)
-{
-	int ret, n;
-	unsigned char omb[4]={
-		0x00, 
-		0x00, 
-		CMD_ClearIMB2, 
-		0x00};
-	unsigned char imb[4];
-	
-	for (n=0; n<insn->n; n++) {
-		if (!(ret=pci1760_mbxrequest(dev, s, omb, imb, OMBCMD_RETRY)))
-			return ret;
-		data[n] = imb[3];
-	}
-
-	return n;
-}
-
 /*
 ==============================================================================
 */
@@ -519,7 +510,7 @@ static int pci1760_insn_bits_do(comedi_device *dev, comedi_subdevice *s,
 		s->state &= ~data[0];
 		s->state |= (data[0]&data[1]);
 		omb[0] = s->state;
-		if (!(ret=pci1760_mbxrequest(dev, s, omb, imb, OMBCMD_RETRY)))
+		if (!(ret=pci1760_mbxrequest(dev, omb, imb)))
 			return ret;
 	}
 	data[1] = s->state;
@@ -542,7 +533,7 @@ static int pci1760_insn_cnt_read(comedi_device *dev, comedi_subdevice *s,
 	unsigned char imb[4];
 
 	for (n=0; n<insn->n; n++) {
-		if (!(ret=pci1760_mbxrequest(dev, s, omb, imb, OMBCMD_RETRY)))
+		if (!(ret=pci1760_mbxrequest(dev, omb, imb)))
 			return ret;
 		data[n] = (imb[1]<<8) + imb[0];
 	}
@@ -567,20 +558,20 @@ static int pci1760_insn_cnt_write(comedi_device *dev, comedi_subdevice *s,
 	unsigned char imb[4];
 
 	if (devpriv->CntResValue[chan] != (data[0]&0xffff)) { // Set reset value if different
-		if (!(ret=pci1760_mbxrequest(dev, s, omb, imb, OMBCMD_RETRY)))
+		if (!(ret=pci1760_mbxrequest(dev, omb, imb)))
 			return ret;
 		devpriv->CntResValue[chan] = data[0]&0xffff;
 	}
 
 	omb[0] = bitmask;	// reset counter to it reset value
 	omb[2] = CMD_ResetIDICounters;
-	if (!(ret=pci1760_mbxrequest(dev, s, omb, imb, OMBCMD_RETRY)))
+	if (!(ret=pci1760_mbxrequest(dev, omb, imb)))
 		return ret;
 
 	if (!(bitmask & devpriv->IDICntEnable)) {	// start counter if it don't run
 		omb[0] = bitmask;
 		omb[2] = CMD_EnableIDICounters;
-		if (!(ret=pci1760_mbxrequest(dev, s, omb, imb, OMBCMD_RETRY)))
+		if (!(ret=pci1760_mbxrequest(dev, omb, imb)))
 			return ret;
 		devpriv->IDICntEnable |= bitmask;
 	}
@@ -604,28 +595,28 @@ static int pci1760_reset(comedi_device *dev)
 
 	omb[0] = 0x00;
 	omb[2] = CMD_SetRelaysOutput;	// reset relay outputs
-	pci1760_mbxrequest(dev, NULL, omb, imb, OMBCMD_RETRY);
+	pci1760_mbxrequest(dev, omb, imb);
 	
 	omb[0] = 0x00;
 	omb[2] = CMD_EnableIDICounters;	// disable IDI up counters
-	pci1760_mbxrequest(dev, NULL, omb, imb, OMBCMD_RETRY);
+	pci1760_mbxrequest(dev, omb, imb);
 	devpriv->IDICntEnable = 0;
 
 	omb[0] = 0x00;
 	omb[2] = CMD_OverflowIDICounters;	// disable counters overflow interrupts
-	pci1760_mbxrequest(dev, NULL, omb, imb, OMBCMD_RETRY);
+	pci1760_mbxrequest(dev, omb, imb);
 	devpriv->IDICntOverEnable = 0;
 
 	omb[0] = 0x00;
 	omb[2] = CMD_MatchIntIDICounters;	// disable counters match value interrupts
-	pci1760_mbxrequest(dev, NULL, omb, imb, OMBCMD_RETRY);
+	pci1760_mbxrequest(dev, omb, imb);
 	devpriv->IDICntMatchEnable = 0;
 
 	omb[0] = 0x00;
 	omb[1] = 0x80;
 	for (i=0; i<8; i++) {	// set IDI up counters match value
 		omb[2] = CMD_SetIDI0CntMatchValue+i;
-		pci1760_mbxrequest(dev, NULL, omb, imb, OMBCMD_RETRY);
+		pci1760_mbxrequest(dev, omb, imb);
 		devpriv->CntMatchValue[i] = 0x8000;
 	}
 
@@ -633,32 +624,32 @@ static int pci1760_reset(comedi_device *dev)
 	omb[1] = 0x00;
 	for (i=0; i<8; i++) {	// set IDI up counters reset value
 		omb[2] = CMD_SetIDI0CntResetValue+i;
-		pci1760_mbxrequest(dev, NULL, omb, imb, OMBCMD_RETRY);
+		pci1760_mbxrequest(dev, omb, imb);
 		devpriv->CntResValue[i] = 0x0000;
 	}
 	
 	omb[0] = 0xff;
 	omb[2] = CMD_ResetIDICounters;	// reset IDI up counters to reset values
-	pci1760_mbxrequest(dev, NULL, omb, imb, OMBCMD_RETRY);
+	pci1760_mbxrequest(dev, omb, imb);
 
 	omb[0] = 0x00;
 	omb[2] = CMD_EdgeIDICounters;	// set IDI up counters count edge
-	pci1760_mbxrequest(dev, NULL, omb, imb, OMBCMD_RETRY);
+	pci1760_mbxrequest(dev, omb, imb);
 	devpriv->IDICntEdge = 0x00;
 
 	omb[0] = 0x00;
 	omb[2] = CMD_EnableIDIFilters;	// disable all digital in filters
-	pci1760_mbxrequest(dev, NULL, omb, imb, OMBCMD_RETRY);
+	pci1760_mbxrequest(dev, omb, imb);
 	devpriv->IDIFiltersEn = 0x00;
 
 	omb[0] = 0x00;
 	omb[2] = CMD_EnableIDIPatternMatch;	// disable pattern matching
-	pci1760_mbxrequest(dev, NULL, omb, imb, OMBCMD_RETRY);
+	pci1760_mbxrequest(dev, omb, imb);
 	devpriv->IDIPatMatchEn = 0x00;
 
 	omb[0] = 0x00;
 	omb[2] = CMD_SetIDIPatternMatch;	// set pattern match value
-	pci1760_mbxrequest(dev, NULL, omb, imb, OMBCMD_RETRY);
+	pci1760_mbxrequest(dev, omb, imb);
 	devpriv->IDIPatMatchValue = 0x00;
 
 	return 0;
@@ -753,8 +744,6 @@ static int pci1760_attach(comedi_device *dev, comedi_devconfig *it)
 	s->maxdata = 1;
 	s->len_chanlist = 8;
 	s->range_table = &range_digital;
-	s->io_bits=0;		/* all bits input */
-	s->insn_read=pci1760_insn_read_di;
 	s->insn_bits=pci1760_insn_bits_di;
 	subdev++;	
 
@@ -765,7 +754,6 @@ static int pci1760_attach(comedi_device *dev, comedi_devconfig *it)
 	s->maxdata = 1;
 	s->len_chanlist = 8;
 	s->range_table = &range_digital;
-	s->io_bits=255;	/* all bits output */
 	s->state=0;
 	s->insn_bits=pci1760_insn_bits_do;
 	subdev++;	
@@ -806,7 +794,6 @@ static int pci_dio_add_di(comedi_device *dev, comedi_subdevice *s,
 	s->maxdata = 1;
 	s->len_chanlist = d->chans;
 	s->range_table = &range_digital;
-	s->io_bits=0;		/* all bits input */
 	switch (this_board->io_access) {
 	case IO_8b:
 		s->insn_bits=pci_dio_insn_bits_di_b;
@@ -833,7 +820,6 @@ static int pci_dio_add_do(comedi_device *dev, comedi_subdevice *s,
 	s->maxdata = 1;
 	s->len_chanlist = d->chans;
 	s->range_table = &range_digital;
-	s->io_bits=(1 << d->chans)-1;	/* all bits output */
 	s->state=0;
 	switch (this_board->io_access) {
 	case IO_8b:
@@ -856,12 +842,7 @@ static int CheckAndAllocCard(comedi_device *dev, comedi_devconfig *it,
 {
 	pci_dio_private	*pr, *prev;
 	
-	if (!pci_priv) {	// well, first card in system
-		pci_priv=devpriv;
-		return 1;
-	}
-	
-	for (pr=pci_priv, prev=NULL; pr!=NULL; prev=pr, pr=pr->next)
+	for (pr=pci_priv, prev=NULL; pr!=NULL; prev=pr, pr=pr->next) {
 		if (pr->pcidev==pcidev) {
 			if (it->options[0]||it->options[1]) {
 				if ((pr->pcidev->bus->number==it->options[0])&&
@@ -872,9 +853,16 @@ static int CheckAndAllocCard(comedi_device *dev, comedi_devconfig *it,
 			}
 			return 0;	// this card is used, look for another
 		}
+	}
 
-	if (prev) { devpriv->prev=prev; }
-		{ pci_priv=devpriv; }
+	if (prev) {
+		devpriv->prev=prev;
+		prev->next = devpriv;
+	} else {
+		pci_priv = devpriv;
+	}
+
+	devpriv->pcidev = pcidev;
 	
 	return 1;
 }
@@ -910,38 +898,28 @@ static int pci_dio_attach(comedi_device *dev, comedi_devconfig *it)
 		ret=CheckAndAllocCard(dev, it, pcidev);	
 		if (ret==1) { found=1; break; }
 		if (ret>1) { 
-			pci_dio_detach(dev);
-			pci_dev_put(pcidev);
 			return -EIO;
 		}
 	}
 
 	if (!found) {
 		rt_printk(", Error: Requested type of the card was not found!\n");
-		pci_dio_detach(dev);
 		return -EIO;
 	}
 	
+	if (pci_enable_device(pcidev)) {
+		rt_printk(", Error: Can't enable PCI device!\n");
+		return -EIO;
+	}
+	if (pci_request_regions(pcidev, driver_pci_dio.driver_name)) {
+		rt_printk(", Error: Can't allocate PCI device!\n");
+		return -EIO;
+	}	
 	iobase=pci_resource_start(pcidev, this_board->main_pci_region);
 	rt_printk(", b:s:f=%d:%d:%d, io=0x%4x",
 		pcidev->bus->number, PCI_SLOT(pcidev->devfn), PCI_FUNC(pcidev->devfn),
 		iobase);
 	
-	if (pci_request_regions(pcidev, driver_pci_dio.driver_name)) {
-		pci_dio_detach(dev);
-		pci_dev_put(pcidev);
-		rt_printk(", Error: Cann't allocate PCI device!\n");
-		return -EIO;
-	}
-	devpriv->pcidev=pcidev;
-	
-	if (pci_enable_device(pcidev)) {
-		pci_dio_detach(dev);
-		rt_printk(", Error: Cann't enable PCI device!\n");
-		return -EIO;
-	}
-	devpriv->enabled=1;
-	
 	dev->iobase=iobase;
 	dev->board_name=this_board->name;
 	
@@ -960,7 +938,6 @@ static int pci_dio_attach(comedi_device *dev, comedi_devconfig *it)
 
         if((ret=alloc_subdevices(dev, n_subdevices))<0) {
 		rt_printk(", Error: Cann't allocate subdevice memory!\n");
-		pci_dio_detach(dev);
     		return ret;
 	}
 
@@ -1047,19 +1024,23 @@ static int pci_dio_detach(comedi_device *dev)
 			s->private=NULL;
 		}
 
-		if (devpriv->enabled)
-			pci_disable_device(devpriv->pcidev);
-
 		if (devpriv->pcidev) {
-			pci_release_regions(devpriv->pcidev);
+			if (dev->iobase) {
+				pci_release_regions(devpriv->pcidev);
+				pci_disable_device(devpriv->pcidev);
+			}
 			pci_dev_put(devpriv->pcidev);
 		}
 		
-		if (devpriv->prev) { devpriv->prev->next=devpriv->next; }
-				    { pci_priv=devpriv->next; }
-		if (devpriv->next) { devpriv->next->prev=devpriv->prev; }
-
-	}	
+		if (devpriv->prev) {
+			devpriv->prev->next=devpriv->next;
+		} else {
+			pci_priv=devpriv->next;
+		}
+		if (devpriv->next) {
+			devpriv->next->prev=devpriv->prev;
+		}
+	}
 
 	return 0;
 }
