@@ -119,7 +119,6 @@ typedef struct{
 	struct pci_dev *pci_dev;
 	int lcr_iobase;	/* PLX PCI9052 config registers in PCIBAR1 */
 	int enable_irq;
-	int share_irq;
 }pc236_private;
 
 #define devpriv ((pc236_private *)dev->private)
@@ -167,15 +166,21 @@ static int pc236_attach(comedi_device *dev,comedi_devconfig *it)
 	comedi_subdevice *s;
 	struct pci_dev *pci_dev = NULL;
 	int iobase = 0, irq = 0;
-	int lcr_iobase = 0;
 	int bus = 0, slot = 0;
 	struct pci_device_id *pci_id;
 	int share_irq = 0;
 	int ret;
 
 	printk("comedi%d: %s: ",dev->minor, PC236_DRIVER_NAME);
-
-	/* Get card bus position and base address. */
+/*
+ * Allocate the private structure area.  alloc_private() is a
+ * convenient macro defined in comedidev.h.
+ */
+	if ((ret=alloc_private(dev,sizeof(pc236_private))) < 0) {
+		printk("out of memory!\n");
+		return ret;
+	}
+	/* Process options. */
 	switch (thisboard->bustype) {
 	case isa_bustype:
 		iobase = it->options[0];
@@ -221,20 +226,13 @@ static int pc236_attach(comedi_device *dev,comedi_devconfig *it)
 			if (((pci_dev->class ^ pci_id->class) & pci_id->class_mask) != 0)
 				continue;
 			/* Found a match. */
+			devpriv->pci_dev = pci_dev;
 			break;
 		}
 		if (!pci_dev) {
 			printk("no %s found!\n", thisboard->fancy_name);
 			return -EIO;
 		}
-		if ((ret=pci_enable_device(pci_dev)) < 0) {
-			printk("error enabling PCI device!\n");
-			pci_dev_put(pci_dev);
-			return ret;
-		}
-		lcr_iobase = pci_resource_start(pci_dev, 1);
-		iobase = pci_resource_start(pci_dev, 2);
-		irq = pci_dev->irq;
 		break;
 	default:
 		printk("bug! cannot determine board type!\n");
@@ -248,29 +246,23 @@ static int pc236_attach(comedi_device *dev,comedi_devconfig *it)
 	dev->board_name = thisboard->name;
 	printk("%s ", dev->board_name);
 
-/*
- * Allocate the private structure area.  alloc_private() is a
- * convenient macro defined in comedidev.h.
- */
-	if ((ret=alloc_private(dev,sizeof(pc236_private))) < 0) {
-		printk("out of memory!\n");
-		if (pci_dev)
-			pci_dev_put(pci_dev);
-		return ret; 
-	}
-
-	devpriv->pci_dev = pci_dev;
-	devpriv->share_irq = share_irq;
-
-	/* Reserve I/O spaces. */
-	if (lcr_iobase) {
-		if ((ret=pc236_request_region(lcr_iobase, PC236_LCR_IO_SIZE)) < 0) {
+	/* Enable device and reserve I/O spaces. */
+	if (pci_dev) {
+		if ((ret=pci_enable_device(pci_dev)) < 0) {
+			printk("error enabling PCI device!\n");
+			return ret;
+		}
+		if ((ret=pci_request_regions(pci_dev, PC236_DRIVER_NAME)) < 0) {
+			printk("I/O port conflict (PCI)!\n");
+			return ret;
+		}
+		devpriv->lcr_iobase = pci_resource_start(pci_dev, 1);
+		iobase = pci_resource_start(pci_dev, 2);
+		irq = pci_dev->irq;
+	} else {
+		if ((ret=pc236_request_region(iobase, PC236_IO_SIZE)) < 0) {
 			return ret;
 		}
-		devpriv->lcr_iobase = lcr_iobase;
-	}
-	if ((ret=pc236_request_region(iobase, PC236_IO_SIZE)) < 0) {
-		return ret;
 	}
 	dev->iobase = iobase;
 
@@ -347,16 +339,18 @@ static int pc236_detach(comedi_device *dev)
 	if (dev->subdevices) {
 		subdev_8255_cleanup(dev, dev->subdevices+0);
 	}
-	if (dev->iobase) {
-		release_region(dev->iobase, PC236_IO_SIZE);
-	}
 	if (devpriv) {
-		if (devpriv->lcr_iobase)
-			release_region(devpriv->lcr_iobase, PC236_LCR_IO_SIZE);
-		if (devpriv->pci_dev)
+		if (devpriv->pci_dev) {
+			if(dev->iobase)
+			{
+				pci_release_regions(devpriv->pci_dev);
+				pci_disable_device(devpriv->pci_dev);
+			}
 			pci_dev_put(devpriv->pci_dev);
+		} else if (dev->iobase) {
+			release_region(dev->iobase, PC236_IO_SIZE);
+		}
 	}
-	
 	return 0;
 }
 
@@ -366,11 +360,10 @@ static int pc236_detach(comedi_device *dev)
  */
 static int pc236_request_region(unsigned long from, unsigned long extent)
 {
-	if (check_region(from, extent) < 0) {
+	if (!from || !request_region(from, extent, PC236_DRIVER_NAME)) {
 		printk("I/O port conflict (%#lx,%lu)!\n", from, extent);
 		return -EIO;
 	}
-	request_region(from, extent, PC236_DRIVER_NAME);
 	return 0;
 }
 
@@ -546,15 +539,14 @@ static irqreturn_t pc236_interrupt(int irq,void *d,struct pt_regs *regs)
 {
 	comedi_device *dev=d;
 	comedi_subdevice *s=dev->subdevices+1;
-	int retval = 1;
+	int handled;
 
-	if(!pc236_intr_check(dev))
-		return IRQ_RETVAL(retval);
-
-	comedi_buf_put(s->async,0);
-	s->async->events |= COMEDI_CB_BLOCK | COMEDI_CB_EOS;
-
-	comedi_event(dev,s,s->async->events);
-	return IRQ_RETVAL(retval);
+	handled = pc236_intr_check(dev);
+	if (handled) {
+		comedi_buf_put(s->async,0);
+		s->async->events |= COMEDI_CB_BLOCK | COMEDI_CB_EOS;
+		comedi_event(dev,s,s->async->events);
+	}
+	return IRQ_RETVAL(handled);
 }
 
