@@ -148,8 +148,15 @@ static int pc263_attach(comedi_device *dev,comedi_devconfig *it)
 	int ret;
 
 	printk("comedi%d: %s: ",dev->minor, PC263_DRIVER_NAME);
-
-	/* Get card bus position and base address. */
+/*
+ * Allocate the private structure area.  alloc_private() is a
+ * convenient macro defined in comedidev.h.
+ */
+	if ((ret=alloc_private(dev,sizeof(pc263_private))) < 0) {
+		printk("out of memory!\n");
+		return ret;
+	}
+	/* Process options. */
 	switch (thisboard->bustype) {
 	case isa_bustype:
 		iobase = it->options[0];
@@ -169,9 +176,9 @@ static int pc263_attach(comedi_device *dev,comedi_devconfig *it)
 		}
 
 		/* Look for matching PCI device. */
-		for(pci_dev = pci_find_device(pci_id->vendor, pci_id->device,
+		for(pci_dev = pci_get_device(pci_id->vendor, pci_id->device,
 					NULL); pci_dev != NULL; 
-				pci_dev = pci_find_device(pci_id->vendor,
+				pci_dev = pci_get_device(pci_id->vendor,
 					pci_id->device, pci_dev)) {
 			/* If bus/slot specified, check them. */
 			if (bus || slot) {
@@ -192,18 +199,13 @@ static int pc263_attach(comedi_device *dev,comedi_devconfig *it)
 			if (((pci_dev->class ^ pci_id->class) & pci_id->class_mask) != 0)
 				continue;
 			/* Found a match. */
+			devpriv->pci_dev = pci_dev;
 			break;
 		}
 		if (!pci_dev) {
 			printk("no %s found!\n", thisboard->fancy_name);
 			return -EIO;
 		}
-		if ((ret=pci_enable_device(pci_dev)) < 0) {
-			printk("error enabling PCI device!\n");
-			pci_dev_put(pci_dev);
-			return ret;
-		}
-		iobase = pci_resource_start(pci_dev, 2);
 		break;
 	default:
 		printk("bug! cannot determine board type!\n");
@@ -217,22 +219,21 @@ static int pc263_attach(comedi_device *dev,comedi_devconfig *it)
 	dev->board_name = thisboard->name;
 	printk("%s ", dev->board_name);
 
-/*
- * Allocate the private structure area.  alloc_private() is a
- * convenient macro defined in comedidev.h.
- */
-	if ((ret=alloc_private(dev,sizeof(pc263_private))) < 0) {
-		printk("out of memory!\n");
-		if (pci_dev)
-			pci_dev_put(pci_dev);
-		return ret;
-	}
-
-	devpriv->pci_dev = pci_dev;
-
-	/* Reserve I/O space. */
-	if ((ret=pc263_request_region(iobase, PC263_IO_SIZE)) < 0) {
-		return ret;
+	/* Enable device and reserve I/O spaces. */
+	if (pci_dev) {
+		if ((ret=pci_enable_device(pci_dev)) < 0) {
+			printk("error enabling PCI device!\n");
+			return ret;
+		}
+		if ((ret=pci_request_regions(pci_dev, PC263_DRIVER_NAME)) < 0) {
+			printk("I/O port conflict (PCI)!\n");
+			return ret;
+		}
+		iobase = pci_resource_start(pci_dev, 2);
+	} else {
+		if ((ret=pc263_request_region(iobase, PC263_IO_SIZE)) < 0) {
+			return ret;
+		}
 	}
 	dev->iobase = iobase;
 
@@ -286,11 +287,19 @@ static int pc263_detach(comedi_device *dev)
 {
 	printk("comedi%d: %s: remove\n", dev->minor, PC263_DRIVER_NAME);
 
-	if (dev->iobase)
-		release_region(dev->iobase, PC263_IO_SIZE);
-	if (devpriv && devpriv->pci_dev)
-		pci_dev_put(devpriv->pci_dev);
-	
+	if (devpriv) {
+		if (devpriv->pci_dev) {
+			if(dev->iobase)
+			{
+				pci_release_regions(devpriv->pci_dev);
+				pci_disable_device(devpriv->pci_dev);
+			}
+			pci_dev_put(devpriv->pci_dev);
+		} else if (dev->iobase) {
+			release_region(dev->iobase, PC263_IO_SIZE);
+		}
+	}
+
 	return 0;
 }
 
@@ -300,11 +309,10 @@ static int pc263_detach(comedi_device *dev)
  */
 static int pc263_request_region(unsigned long from, unsigned long extent)
 {
-	if (check_region(from, extent) < 0) {
+	if (!request_region(from, extent, PC263_DRIVER_NAME)) {
 		printk("I/O port conflict (%#lx,%lu)!\n", from, extent);
 		return -EIO;
 	}
-	request_region(from, extent, PC263_DRIVER_NAME);
 	return 0;
 }
 
