@@ -74,9 +74,9 @@ extra triggered scan functionality, interrupt bug-fix added by Steve Sharples
 #define PCI230_ADCG    0x0E
 
 /* Convertor related constants. */
-#define PCI230_DAC_SETTLE 5		/* Analogue output settling time in s (DAC itself is 1s nominally). */  
-#define PCI230_ADC_SETTLE 1		/* Analogue input settling time in s (ADC itself is 1.6s nominally but we poll anyway). */  
-#define PCI230_MUX_SETTLE 10	/* ADC MUX settling time in S - 10s for se, 20s de. */
+#define PCI230_DAC_SETTLE 5		/* Analogue output settling time in µs (DAC itself is 1µs nominally). */  
+#define PCI230_ADC_SETTLE 1		/* Analogue input settling time in µs (ADC itself is 1.6µs nominally but we poll anyway). */  
+#define PCI230_MUX_SETTLE 10	/* ADC MUX settling time in µS - 10µs for se, 20µs de. */
 
 /* DACCON values. */
 #define PCI230_DAC_BUSY_BIT		1
@@ -265,14 +265,20 @@ static int pci230_ao_winsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *i
 static int pci230_ao_rinsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data);
 static int pci230_ct_insn_config(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data);
 static int pci230_ct_rinsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data);
+#if 0
 static void pci230_ns_to_timer(unsigned int *ns,int round);
+#endif
 static void pci230_ns_to_single_timer(unsigned int *ns,int round);
 static void i8253_single_ns_to_timer(unsigned int i8253_osc_base, unsigned int *d, unsigned int *nanosec, int round_mode);
 static void pci230_setup_monostable_ct0(comedi_device *dev, unsigned int ns, unsigned int n_chan);
+#if 0
 static void pci230_z2_ct0(comedi_device *dev, unsigned int *ns,int round);
+#endif
 static void pci230_z2_ct1(comedi_device *dev, unsigned int *ns,int round);
 static void pci230_z2_ct2(comedi_device *dev, unsigned int *ns,int round);
+#if 0
 static void pci230_cancel_ct0(comedi_device *dev);
+#endif
 static void pci230_cancel_ct1(comedi_device *dev);
 static void pci230_cancel_ct2(comedi_device *dev);
 static irqreturn_t pci230_interrupt(int irq, void *d, struct pt_regs *regs);
@@ -331,6 +337,11 @@ static int pci230_attach(comedi_device *dev,comedi_devconfig *it)
 
 	printk("comedi%d: amplc_pci230\n",dev->minor);
 	
+	/* Allocate the private structure area using alloc_private().
+	 * Macro defined in comedidev.h - memsets struct fields to 0. */
+	if((alloc_private(dev,sizeof(struct pci230_private)))<0){
+		return -ENOMEM;
+	}
 	/* Find card */
 	for(pci_dev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pci_dev != NULL ; 
 		pci_dev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pci_dev)) {
@@ -347,41 +358,26 @@ static int pci230_attach(comedi_device *dev,comedi_devconfig *it)
 		printk("comedi%d: amplc_pci230: No PCI230 found\n",dev->minor);
 		return -EIO;
 	}
+	devpriv->pci_dev = pci_dev;
 	dev->board_ptr = pci230_boards+i;
 	
 	/* Read base addressses of the PCI230's two I/O regions from PCI configuration register. */
 	if(pci_enable_device(pci_dev)<0){
-		pci_dev_put(pci_dev);
 		return -EIO;
 	}
 
 	pci_iobase = pci_resource_start(pci_dev, 2);
 	iobase = pci_resource_start(pci_dev, 3);
 
+	/* Reserve I/O spaces. */
+	if(pci_request_regions(pci_dev,"PCI230")<0){
+		printk("comedi%d: amplc_pci230: I/O space conflict\n",dev->minor);
+		return -EIO;
+	}
+
 	printk("comedi%d: amplc_pci230: I/O region 1 0x%04x I/O region 2 0x%04x\n",dev->minor, pci_iobase, iobase);
 
-	/* Allocate the private structure area using alloc_private().
-	 * Macro defined in comedidev.h - memsets struct fields to 0. */
-	if((alloc_private(dev,sizeof(struct pci230_private)))<0){
-		pci_dev_put(pci_dev);
-		return -ENOMEM;
-	}
-	devpriv->pci_dev = pci_dev;
-
-	/* Reserve I/O space 1. */
-	if(check_region(pci_iobase,PCI230_IO1_SIZE)<0){
-		printk("comedi%d: amplc_pci230: I/O space 1 conflict\n",dev->minor);
-		return -EIO;
-	}
-	request_region(pci_iobase,PCI230_IO1_SIZE,"PCI230");
 	devpriv->pci_iobase = pci_iobase;
-
-	/* Reserve I/O space 2. */
-	if(check_region(iobase,PCI230_IO2_SIZE)<0){
-		printk("comedi%d: amplc_pci230: I/O space 2 conflict\n",dev->minor);
-		return -EIO;
-	}
-	request_region(iobase,PCI230_IO2_SIZE,"PCI230");
 	dev->iobase = iobase;
 
 /*
@@ -390,6 +386,9 @@ static int pci230_attach(comedi_device *dev,comedi_devconfig *it)
  */
 	dev->board_name = thisboard->name;
 
+	/* Disable board's interrupts. */
+	outb(0, devpriv->pci_iobase + PCI230_INT_SCE);
+
 	/* Register the interrupt handler. */
 	irq_hdl = comedi_request_irq(devpriv->pci_dev->irq, pci230_interrupt, SA_SHIRQ, "amplc_pci230", dev);
 	if(irq_hdl<0) {
@@ -399,7 +398,7 @@ static int pci230_attach(comedi_device *dev,comedi_devconfig *it)
 		dev->irq = devpriv->pci_dev->irq;
 		printk("comedi%d: amplc_pci230: registered irq %d\n", dev->minor, devpriv->pci_dev->irq);
 	}
-
+	
 /*
  * Allocate the subdevice structures.  alloc_subdevice() is a
  * convenient macro defined in comedidev.h.
@@ -481,17 +480,16 @@ static int pci230_detach(comedi_device *dev)
 	if(dev->subdevices && thisboard->have_dio)
 		subdev_8255_cleanup(dev,dev->subdevices + 2);	/* Clean up dio subdevice. */
 
-	if(dev->iobase)
-		release_region(dev->iobase,PCI230_IO2_SIZE);
-
 	if(dev->irq)
 		comedi_free_irq(dev->irq, dev);
 
 	if(devpriv){
-		if(devpriv->pci_iobase){
-			release_region(devpriv->pci_iobase, PCI230_IO1_SIZE);
-		}
 		if(devpriv->pci_dev){
+			if(dev->iobase)
+			{
+				pci_release_regions(devpriv->pci_dev);
+				pci_disable_device(devpriv->pci_dev);
+			}
 			pci_dev_put(devpriv->pci_dev);
 		}
 	}
@@ -562,7 +560,7 @@ static int pci230_ai_rinsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *i
 		/* wait for conversion to end */
 		for(i=0;i<TIMEOUT;i++){
 			status = inw(dev->iobase + PCI230_ADCCON);
-			if(PCI230_TEST_BIT(status, PCI230_ADC_BUSY_BIT))break;
+			if(!PCI230_TEST_BIT(status, PCI230_ADC_BUSY_BIT)) break;
 		}
 		if(i==TIMEOUT){
 			/* rt_printk() should be used instead of printk()
@@ -1157,6 +1155,7 @@ static int pci230_ai_cmd(comedi_device *dev,comedi_subdevice *s)
 }
 
 
+#if 0
 /* This function doesn't require a particular form, this is just
  * what happens to be used in some of the drivers.  It should
  * convert ns nanoseconds to a counter value suitable for programming
@@ -1168,6 +1167,7 @@ static void pci230_ns_to_timer(unsigned int *ns,int round)
 	i8253_cascade_ns_to_timer_2div(PCI230_TIMEBASE_10MHZ, &divisor0, &divisor1, ns, TRIG_ROUND_MASK);
 	return;
 }
+#endif
 
 
 /* This function is used for analogue input. Only one counter/timer can be used,
@@ -1203,7 +1203,6 @@ static void pci230_ns_to_single_timer(unsigned int *ns,int round)
 
 static void i8253_single_ns_to_timer(unsigned int i8253_osc_base, unsigned int *d, unsigned int *nanosec, int round_mode)
 {
-        int divider;
         unsigned int div;
 
         /* exit early if everything is already correct (this can save time
@@ -1274,6 +1273,7 @@ static void pci230_setup_monostable_ct0(comedi_device *dev, unsigned int ns, uns
 }
 
 
+#if 0
 /* 
  *  Set ZCLK_CT0 to square wave mode with period of ns.
  */
@@ -1306,12 +1306,15 @@ static void pci230_z2_ct0(comedi_device *dev, unsigned int *ns,int round)
 	}
 	return;
 }
+#endif
 
+#if 0
 static void pci230_cancel_ct0(comedi_device *dev)
 {
 	devpriv->divisor0 = 0;
 	i8254_load(devpriv->pci_iobase + PCI230_Z2_CT0, 0, devpriv->divisor0, 0);	/* Counter 0, divisor0, 8254 mode 0. */
 }
+#endif
 
 /* 
  *  Set ZCLK_CT1 to square wave mode with period of ns.
@@ -1399,7 +1402,6 @@ static irqreturn_t pci230_interrupt(int irq, void *d, struct pt_regs *regs)
 	int status_int;
 	comedi_device *dev = (comedi_device*) d;
 	comedi_subdevice *s;
-	int retval = 1;
 
 	/* Read interrupt status/enable register. */
 	status_int = inb(devpriv->pci_iobase + PCI230_INT_SCE);
@@ -1436,7 +1438,7 @@ static irqreturn_t pci230_interrupt(int irq, void *d, struct pt_regs *regs)
 		s->async->events = 0;
 	}
 
-	return IRQ_RETVAL(retval);
+	return IRQ_HANDLED;
 }
 
 static void pci230_handle_ao(comedi_device *dev, comedi_subdevice *s) {
@@ -1505,13 +1507,11 @@ static void pci230_handle_ai(comedi_device *dev, comedi_subdevice *s) {
 		/* Cancel sampled conversion. */
 		s->async->events |= COMEDI_CB_ERROR | COMEDI_CB_EOA;
 		pci230_ai_cancel(dev, s);	
-	}
-	if(devpriv->ai_count == 0 && devpriv->ai_stop == 0) {
+	}else if(devpriv->ai_count == 0 && devpriv->ai_stop == 0) {
 		/* Acquisition complete. */
 		s->async->events |= COMEDI_CB_EOA;
 		pci230_ai_cancel(dev, s);			/* disable hardware conversions */
-	}
-	else {
+	}else {
 		/* More samples required, tell Comedi to block. */
 		s->async->events |= COMEDI_CB_BLOCK;
 
