@@ -35,7 +35,6 @@ Status: experimental
 #include <linux/delay.h>
 #include <linux/pci.h>
 
-#include <pcmcia/version.h>
 #include <pcmcia/cs_types.h>
 #include <pcmcia/cs.h>
 #include <pcmcia/cistpl.h>
@@ -481,32 +480,30 @@ static int das16cs_ao_winsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *
 		devpriv->ao_readback[chan] = data[i];
 		d = data[i];
 
-		//devpriv->status1 |= 0x0009;
 		outw(devpriv->status1, dev->iobase + 4);
 		comedi_udelay(1);
 
-		status1 = devpriv->status1;
-		//if(chan)status1 &= ~0x0008;
-		//else status1 &= ~0x0001;
-		if(chan)status1 |= 0x0008;
-		else status1 |= 0x0001;
+		status1 = devpriv->status1 & ~0xf;
+		if(chan) status1 |= 0x0001;
+		else status1 |= 0x0008;
 
-		printk("0x%04x\n",status1);
+/* 		printk("0x%04x\n",status1);*/
 		outw(status1, dev->iobase + 4);
 		comedi_udelay(1);
 
 		for(bit=15;bit>=0;bit--){
-			int b = (d>>bit)&1;
-
-			printk("0x%04x\n",status1 | b | 0x0000);
+			int b = (d >> bit) & 0x1;
+			b <<= 1;
+/*			printk("0x%04x\n",status1 | b | 0x0000);*/
 			outw(status1 | b | 0x0000, dev->iobase + 4);
 			comedi_udelay(1);
-			printk("0x%04x\n",status1 | b | 0x0004);
+/*			printk("0x%04x\n",status1 | b | 0x0004);*/
 			outw(status1 | b | 0x0004, dev->iobase + 4);
 			comedi_udelay(1);
 		}
-
-		outw(devpriv->status1, dev->iobase + 4);
+/*		make high both DAC0CS and DAC1CS to load
+		new data and update analog output*/
+		outw(status1 | 0x9, dev->iobase + 4);
 	}
 
 	return i;
@@ -623,7 +620,7 @@ static int das16cs_timer_insn_config(comedi_device *dev,comedi_subdevice *s,
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
-MODULE_PARM(pc_debug, "i");
+module_param(pc_debug, int, 0644);
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
 "cb_das16_cs.c pcmcia code (David Schleef), modified from dummy_cs.c 1.31 2001/08/24 12:13:13 (David Hinds)";
@@ -633,32 +630,10 @@ static char *version =
 
 /*====================================================================*/
 
-/* Parameters that can be set with 'insmod' */
-
-/* The old way: bit map of interrupts to choose from */
-/* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
-static u_int irq_mask = 0xdeb8;
-/* Newer, simpler way of listing specific interrupts */
-static int irq_list[4] = { -1 };
-
-MODULE_PARM(irq_mask, "i");
-MODULE_PARM(irq_list, "1-4i");
-
-/*====================================================================*/
-
-/*
-   The event() function is this driver's Card Services event handler.
-   It will be called by Card Services when an appropriate card status
-   event is received.  The config() and release() entry points are
-   used to configure or release a socket, in response to card
-   insertion and ejection events.  They are invoked from the das08_pcmcia
-   event handler.
-*/
-
 static void das16cs_pcmcia_config(dev_link_t *link);
 static void das16cs_pcmcia_release(u_long arg);
-static int das16cs_pcmcia_event(event_t event, int priority,
-		       event_callback_args_t *args);
+static int das16cs_pcmcia_suspend(struct pcmcia_device *p_dev);
+static int das16cs_pcmcia_resume(struct pcmcia_device *p_dev);
 
 /*
    The attach() and detach() entry points are used to create and destroy
@@ -666,8 +641,8 @@ static int das16cs_pcmcia_event(event_t event, int priority,
    needed to manage one actual PCMCIA card.
 */
 
-static dev_link_t *das16cs_pcmcia_attach(void);
-static void das16cs_pcmcia_detach(dev_link_t *);
+static int das16cs_pcmcia_attach(struct pcmcia_device *);
+static void das16cs_pcmcia_detach(struct pcmcia_device *);
 
 /*
    You'll also need to prototype all the functions that will actually
@@ -732,34 +707,23 @@ typedef struct local_info_t {
 
 ======================================================================*/
 
-static dev_link_t *das16cs_pcmcia_attach(void)
+static int das16cs_pcmcia_attach(struct pcmcia_device *p_dev)
 {
     local_info_t *local;
     dev_link_t *link;
-    client_reg_t client_reg;
-    int ret, i;
 
     DEBUG(0, "das16cs_pcmcia_attach()\n");
 
     /* Allocate space for private device-specific data */
     local = kmalloc(sizeof(local_info_t), GFP_KERNEL);
-    if (!local) return NULL;
+    if (!local) return -ENOMEM;
     memset(local, 0, sizeof(local_info_t));
     link = &local->link; link->priv = local;
 
     /* Initialize the dev_link_t structure */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-	link->release.function = &das16cs_pcmcia_release;
-    link->release.data = (u_long)link;
-#endif
     /* Interrupt setup */
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
-    link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-    if (irq_list[0] == -1)
-	link->irq.IRQInfo2 = irq_mask;
-    else
-	for (i = 0; i < 4; i++)
-	    link->irq.IRQInfo2 |= 1 << irq_list[i];
+    link->irq.IRQInfo1 = IRQ_LEVEL_ID;
     link->irq.Handler = NULL;
 
     /*
@@ -773,26 +737,15 @@ static dev_link_t *das16cs_pcmcia_attach(void)
     link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
 
-    /* Register with Card Services */
     link->next = dev_list;
     dev_list = link;
-    client_reg.dev_info = &dev_info;
-    client_reg.Attributes = INFO_IO_CLIENT | INFO_CARD_SHARE;
-    client_reg.EventMask =
-	CS_EVENT_CARD_INSERTION | CS_EVENT_CARD_REMOVAL |
-	CS_EVENT_RESET_PHYSICAL | CS_EVENT_CARD_RESET |
-	CS_EVENT_PM_SUSPEND | CS_EVENT_PM_RESUME;
-    client_reg.event_handler = &das16cs_pcmcia_event;
-    client_reg.Version = 0x0210;
-    client_reg.event_callback_args.client_data = link;
-    ret = pcmcia_register_client(&link->handle, &client_reg);
-    if (ret != CS_SUCCESS) {
-	cs_error(link->handle, RegisterClient, ret);
-	das16cs_pcmcia_detach(link);
-	return NULL;
-    }
 
-    return link;
+    link->handle = p_dev;
+    p_dev->instance = link;
+    link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
+    das16cs_pcmcia_config(link);
+
+    return 0;
 } /* das16cs_pcmcia_attach */
 
 /*======================================================================
@@ -804,8 +757,9 @@ static dev_link_t *das16cs_pcmcia_attach(void)
 
 ======================================================================*/
 
-static void das16cs_pcmcia_detach(dev_link_t *link)
+static void das16cs_pcmcia_detach(struct pcmcia_device *p_dev)
 {
+	dev_link_t *link = dev_to_instance(p_dev);
 	dev_link_t **linkp;
 
 	DEBUG(0, "das16cs_pcmcia_detach(0x%p)\n", link);
@@ -814,7 +768,7 @@ static void das16cs_pcmcia_detach(dev_link_t *link)
 	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
 	if (*linkp == link) break;
 	if (*linkp == NULL)
-	return;
+		return;
 
 	/*
 	If the device is currently configured and active, we won't
@@ -824,23 +778,13 @@ static void das16cs_pcmcia_detach(dev_link_t *link)
 	*/
 	if (link->state & DEV_CONFIG)
 	{
-#ifdef PCMCIA_DEBUG
-		printk(KERN_DEBUG "das16cs: detach postponed, '%s' "
-			"still locked\n", link->dev->dev_name);
-#endif
-		link->state |= DEV_STALE_LINK;
-		return;
+		((local_info_t *)link->priv)->stop = 1;
+		das16cs_pcmcia_release((u_long)link);
 	}
-
-	/* Break the link with Card Services */
-	if (link->handle)
-		pcmcia_deregister_client(link->handle);
-
 	/* Unlink device structure, and free it */
 	*linkp = link->next;
 	/* This points to the parent local_info_t struct */
 	kfree(link->priv);
-
 } /* das16cs_pcmcia_detach */
 
 /*======================================================================
@@ -860,7 +804,6 @@ static void das16cs_pcmcia_config(dev_link_t *link)
 	int last_fn, last_ret;
 	u_char buf[64];
 	config_info_t conf;
-	win_req_t req;
 	cistpl_cftable_entry_t dflt = { 0 };
 
 	DEBUG(0, "das16cs_pcmcia_config(0x%p)\n", link);
@@ -1013,9 +956,6 @@ static void das16cs_pcmcia_config(dev_link_t *link)
 	if (link->io.NumPorts2)
 	printk(" & 0x%04x-0x%04x", link->io.BasePort2,
 			link->io.BasePort2+link->io.NumPorts2-1);
-	if (link->win)
-	printk(", mem 0x%06lx-0x%06lx", req.Base,
-			req.Base+req.Size-1);
 	printk("\n");
 
 	link->state &= ~DEV_CONFIG_PENDING;
@@ -1041,19 +981,6 @@ static void das16cs_pcmcia_release(u_long arg)
 
 	DEBUG(0, "das16cs_pcmcia_release(0x%p)\n", link);
 
-    /*
-       If the device is currently in use, we won't release until it
-       is actually closed, because until then, we can't be sure that
-       no one will try to access the device or its data structures.
-    */
-	if (link->open)
-	{
-		DEBUG(1, "das16cs: release postponed, '%s' still open\n",
-			link->dev->dev_name);
-		link->state |= DEV_STALE_CONFIG;
-		return;
-	}
-
 	/* Unlink the device chain */
 	link->dev = NULL;
 
@@ -1063,109 +990,65 @@ static void das16cs_pcmcia_release(u_long arg)
     */
 
     /* Don't bother checking to see if these succeed or not */
-	if (link->win)
-		pcmcia_release_window(link->win);
 	pcmcia_release_configuration(link->handle);
 	if (link->io.NumPorts1)
 		pcmcia_release_io(link->handle, &link->io);
 	if (link->irq.AssignedIRQ)
 		pcmcia_release_irq(link->handle, &link->irq);
 	link->state &= ~DEV_CONFIG;
-
-	if (link->state & DEV_STALE_LINK)
-		das16cs_pcmcia_detach(link);
-
 } /* das16cs_pcmcia_release */
 
-/*======================================================================
-
-    The card status event handler.  Mostly, this schedules other
-    stuff to run after an event is received.
-
-    When a CARD_REMOVAL event is received, we immediately set a
-    private flag to block future accesses to this device.  All the
-    functions that actually access the device should check this flag
-    to make sure the card is still present.
-
-======================================================================*/
-
-static int das16cs_pcmcia_event(event_t event, int priority,
-	event_callback_args_t *args)
+static int das16cs_pcmcia_suspend(struct pcmcia_device *p_dev)
 {
-	dev_link_t *link = args->client_data;
-	local_info_t *dev = link->priv;
+	dev_link_t *link = dev_to_instance(p_dev);
+	local_info_t *local = link->priv;
 
-	DEBUG(1, "das16cs_pcmcia_event(0x%06x)\n", event);
-
-	switch (event)
-	{
-		case CS_EVENT_CARD_REMOVAL:
-			link->state &= ~DEV_PRESENT;
-			if (link->state & DEV_CONFIG)
-			{
-				((local_info_t *)link->priv)->stop = 1;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-				mod_timer(&link->release, jiffies + HZ/20);
-#else
-				das16cs_pcmcia_release((u_long)link);
-#endif
-			}
-			break;
-		case CS_EVENT_CARD_INSERTION:
-			link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-			dev->bus = args->bus;
-#endif
-			das16cs_pcmcia_config(link);
-			break;
-		case CS_EVENT_PM_SUSPEND:
-			link->state |= DEV_SUSPEND;
-			/* Fall through... */
-		case CS_EVENT_RESET_PHYSICAL:
-			/* Mark the device as stopped, to block IO until later */
-			dev->stop = 1;
-			if (link->state & DEV_CONFIG)
-			pcmcia_release_configuration(link->handle);
-			break;
-		case CS_EVENT_PM_RESUME:
-			link->state &= ~DEV_SUSPEND;
-			/* Fall through... */
-		case CS_EVENT_CARD_RESET:
-			if (link->state & DEV_CONFIG)
-				pcmcia_request_configuration(link->handle, &link->conf);
-			dev->stop = 0;
-/*
-In a normal driver, additional code may go here to restore
-the device state and restart IO.
-*/
-			break;
-	}
+	link->state |= DEV_SUSPEND;
+	/* Mark the device as stopped, to block IO until later */
+	local->stop = 1;
+	if (link->state & DEV_CONFIG)
+		pcmcia_release_configuration(link->handle);
 
 	return 0;
-} /* das16cs_pcmcia_event */
+} /* das16cs_pcmcia_suspend */
+
+static int das16cs_pcmcia_resume(struct pcmcia_device *p_dev)
+{
+	dev_link_t *link = dev_to_instance(p_dev);
+	local_info_t *local = link->priv;
+
+	link->state &= ~DEV_SUSPEND;
+	if (link->state & DEV_CONFIG)
+		pcmcia_request_configuration(link->handle, &link->conf);
+	local->stop = 0;
+	return 0;
+} /* das16cs_pcmcia_resume */
 
 /*====================================================================*/
+
+static struct pcmcia_device_id das16cs_id_table[] =
+{
+	PCMCIA_DEVICE_MANF_CARD(0x01c5, 0x0039),
+	PCMCIA_DEVICE_NULL
+};
+MODULE_DEVICE_TABLE(pcmcia, das16cs_id_table);
+
 struct pcmcia_driver das16cs_driver =
 {
-	.attach = das16cs_pcmcia_attach,
-	.detach = das16cs_pcmcia_detach,
+	.probe = das16cs_pcmcia_attach,
+	.remove = das16cs_pcmcia_detach,
+	.suspend = das16cs_pcmcia_suspend,
+	.resume = das16cs_pcmcia_resume,
+	.id_table = das16cs_id_table,
 	.owner = THIS_MODULE,
 	.drv = {
-		.name = "cb_das16_cs",
+		.name = dev_info,
 	},	
 };
 
 static int __init init_das16cs_pcmcia_cs(void)
 {
-	servinfo_t serv;
 	DEBUG(0, "%s\n", version);
-	pcmcia_get_card_services_info(&serv);
-	if (serv.Revision != CS_RELEASE_CODE)
-	{
-		printk(KERN_NOTICE "das16cs: Card Services release "
-			"does not match!\n");
-		//return -1;
-	}
 	pcmcia_register_driver(&das16cs_driver);
 	return 0;
 }
@@ -1176,16 +1059,13 @@ static void __exit exit_das16cs_pcmcia_cs(void)
 	pcmcia_unregister_driver(&das16cs_driver);
 	while (dev_list != NULL)
 	{
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-		del_timer(&dev_list->release);
-#endif
 		if (dev_list->state & DEV_CONFIG)
 			das16cs_pcmcia_release((u_long)dev_list);
-		das16cs_pcmcia_detach(dev_list);
+		das16cs_pcmcia_detach(dev_list->handle);
 	}
 }
 
-int init_module(void)
+int __init init_module(void)
 {
 	int ret;
 
@@ -1196,7 +1076,7 @@ int init_module(void)
 	return comedi_driver_register(&driver_das16cs);
 }
 
-void cleanup_module(void)
+void __exit cleanup_module(void)
 {
 	exit_das16cs_pcmcia_cs();
 	comedi_driver_unregister(&driver_das16cs);
