@@ -507,9 +507,6 @@ static int cb_pcidas_attach(comedi_device *dev, comedi_devconfig *it)
 	comedi_subdevice *s;
 	struct pci_dev* pcidev;
 	int index;
-	unsigned long s5933_config, control_status, adc_fifo,
-		pacer_counter_dio, ao_registers;
-	int err;
 	int i;
 
 	printk("comedi%d: cb_pcidas: ",dev->minor);
@@ -525,8 +522,8 @@ static int cb_pcidas_attach(comedi_device *dev, comedi_devconfig *it)
  */
 	printk("\n");
 
-	for(pcidev = pci_find_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ; 
-		pcidev = pci_find_device(PCI_ANY_ID, PCI_ANY_ID, pcidev)) 
+	for(pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ; 
+		pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pcidev)) 
 	{
 		// is it not a computer boards card?
 		if(pcidev->vendor != PCI_VENDOR_ID_CB)
@@ -559,50 +556,32 @@ static int cb_pcidas_attach(comedi_device *dev, comedi_devconfig *it)
 found:
 
 	printk("Found %s on bus %i, slot %i\n", cb_pcidas_boards[index].name,
-		devpriv->pci_dev->bus->number, PCI_SLOT(devpriv->pci_dev->devfn));
+		pcidev->bus->number, PCI_SLOT(pcidev->devfn));
 
 	/*
-	 * Initialize devpriv->control_status and devpriv->adc_fifo to point to
-	 * their base address.
+	 * Enable PCI device and reserve I/O ports.
 	 */
-	if(pci_enable_device(devpriv->pci_dev))
+	if(pci_enable_device(pcidev))
+	{
+		printk(" Failed to enable PCI device\n");
 		return -EIO;
-	s5933_config = pci_resource_start(devpriv->pci_dev, S5933_BADRINDEX); 
-	control_status = pci_resource_start(devpriv->pci_dev, CONT_STAT_BADRINDEX); 
-	adc_fifo = pci_resource_start(devpriv->pci_dev, ADC_FIFO_BADRINDEX); 
-	pacer_counter_dio = pci_resource_start(devpriv->pci_dev, PACER_BADRINDEX); 
-	ao_registers = pci_resource_start(devpriv->pci_dev, AO_BADRINDEX); 
-
-	// reserve io ports
-	err = 0;
-	if(check_region(s5933_config, AMCC_OP_REG_SIZE) < 0)
-		err++;
-	if(check_region(control_status, CONT_STAT_SIZE) < 0)
-		err++;
-	if(check_region(adc_fifo, ADC_FIFO_SIZE) < 0)
-		err++;
-	if(check_region(pacer_counter_dio, PACER_SIZE) < 0)
-		err++;
-	if(thisboard->ao_nchan)
-		if(check_region(ao_registers, AO_SIZE) < 0)
-			err++;
-	if(err)
+	}
+	if(pci_request_regions(pcidev, "cb_pcidas"))
 	{
 		printk(" I/O port conflict\n");
 		return -EIO;
 	}
-	request_region(s5933_config, AMCC_OP_REG_SIZE, "cb_pcidas");
-	devpriv->s5933_config = s5933_config;
-	request_region(control_status, CONT_STAT_SIZE, "cb_pcidas");
-	devpriv->control_status = control_status;
-	request_region(adc_fifo, ADC_FIFO_SIZE, "cb_pcidas");
-	devpriv->adc_fifo = adc_fifo;
-	request_region(pacer_counter_dio, PACER_SIZE, "cb_pcidas");
-	devpriv->pacer_counter_dio = pacer_counter_dio;
+	/*
+	 * Initialize devpriv->control_status and devpriv->adc_fifo to point to
+	 * their base address.
+	 */
+	devpriv->s5933_config = pci_resource_start(devpriv->pci_dev, S5933_BADRINDEX); 
+	devpriv->control_status = pci_resource_start(devpriv->pci_dev, CONT_STAT_BADRINDEX); 
+	devpriv->adc_fifo = pci_resource_start(devpriv->pci_dev, ADC_FIFO_BADRINDEX); 
+	devpriv->pacer_counter_dio = pci_resource_start(devpriv->pci_dev, PACER_BADRINDEX); 
 	if(thisboard->ao_nchan)
 	{
-		request_region(ao_registers, AO_SIZE, "cb_pcidas");
-		devpriv->ao_registers = ao_registers;
+		devpriv->ao_registers = pci_resource_start(devpriv->pci_dev, AO_BADRINDEX); 
 	}
 
 	// get irq
@@ -753,23 +732,21 @@ static int cb_pcidas_detach(comedi_device *dev)
 #ifdef CB_PCIDAS_DEBUG
 			rt_printk("detaching, incsr is 0x%x\n", inl(devpriv->s5933_config + AMCC_OP_REG_INTCSR));
 #endif
-			release_region(devpriv->s5933_config, AMCC_OP_REG_SIZE);
 		}
-		if(devpriv->control_status)
-			release_region(devpriv->control_status, CONT_STAT_SIZE);
-		if(devpriv->adc_fifo)
-			release_region(devpriv->adc_fifo, ADC_FIFO_SIZE);
-		if(devpriv->pacer_counter_dio)
-			release_region(devpriv->pacer_counter_dio, PACER_SIZE);
-		if(devpriv->ao_registers)
-			release_region(devpriv->ao_registers, AO_SIZE);
 	}
 	if(dev->irq)
 		comedi_free_irq(dev->irq, dev);
 	if(dev->subdevices)
 		subdev_8255_cleanup(dev,dev->subdevices + 2);
 	if(devpriv && devpriv->pci_dev)
+	{
+		if(devpriv->s5933_config)
+		{
+			pci_release_regions(devpriv->pci_dev);
+			pci_disable_device(devpriv->pci_dev);
+		}
 		pci_dev_put(devpriv->pci_dev);
+	}
 
 	return 0;
 }
@@ -1525,7 +1502,6 @@ static irqreturn_t cb_pcidas_interrupt(int irq, void *d, struct pt_regs *regs)
 	unsigned int num_samples, i;
 	static const int timeout = 10000;
 	unsigned long flags;
-	int retval = 1;
 
 	if(dev->attached == 0)
 	{
@@ -1630,7 +1606,7 @@ static irqreturn_t cb_pcidas_interrupt(int irq, void *d, struct pt_regs *regs)
 
 	comedi_event(dev, s, async->events);
 
-	return IRQ_RETVAL(retval);
+	return IRQ_HANDLED;
 }
 
 static void handle_ao_interrupt(comedi_device *dev, unsigned int status)
