@@ -288,31 +288,22 @@ static int cb_pcidda_attach(comedi_device *dev, comedi_devconfig *it)
 		pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pcidev)) {
 		if(pcidev->vendor==PCI_VENDOR_ID_CB){
 			if(it->options[0] || it->options[1]){
-				if(pcidev->bus->number==it->options[0] &&
-				   PCI_SLOT(pcidev->devfn)==it->options[1]){
-					break;
+				if(pcidev->bus->number != it->options[0] ||
+				   PCI_SLOT(pcidev->devfn) != it->options[1]){
+					continue;
+				}
+			}
+			for(index=0;index<N_BOARDS;index++){
+				if(cb_pcidda_boards[index].device_id == pcidev->device){
+					goto found;
 				}
-			}else{
-				break;
 			}
 		}
 	}
-
 	if(!pcidev){
 		printk("Not a ComputerBoards/MeasurementComputing card on requested position\n");
 		return -EIO;
 	}
-
-	for(index=0;index<N_BOARDS;index++){
-		if(cb_pcidda_boards[index].device_id == pcidev->device){
-			goto found;
-		}
-	}
-	printk("Not a supported ComputerBoards/MeasurementComputing card on "
-		"requested position\n");
-	pci_dev_put(pcidev);
-	return -EIO;
-
 found:
 	devpriv->pci_dev = pcidev;
 	dev->board_ptr = cb_pcidda_boards+index;
@@ -323,13 +314,16 @@ found:
 	 * Initialize devpriv->control_status and devpriv->adc_fifo to point to
 	 * their base address.
 	 */
-	if(pci_enable_device(devpriv->pci_dev))
+	if(pci_enable_device(pcidev))
+	{
+		printk("cb_pcidda: failed to enable PCI device\n");
 		return -EIO;
+	}
 
 /*
  * Allocate the I/O ports.
  */
-	if (pci_request_regions(devpriv->pci_dev, thisboard->name))
+	if (pci_request_regions(pcidev, thisboard->name))
 	{
 		printk("cb_pcidda: I/O port conflict\n");
 		return -EIO;
@@ -405,12 +399,15 @@ static int cb_pcidda_detach(comedi_device *dev)
  */
 	if(devpriv)
 	{
-		if(devpriv->digitalio)
-			release_region(devpriv->digitalio, DIGITALIO_SIZE);
-		if(devpriv->dac)
-			release_region(devpriv->dac, 8 + thisboard->ao_chans*2);
 		if(devpriv->pci_dev)
+		{
+			if(devpriv->dac)
+			{
+				pci_release_regions(devpriv->pci_dev);
+				pci_disable_device(devpriv->pci_dev);
+			}
 			pci_dev_put(devpriv->pci_dev);
+		}
 	}
 	// cleanup 8255
 	if(dev->subdevices)
