@@ -1171,6 +1171,41 @@ static void das16_timer_interrupt(unsigned long arg)
 		mod_timer(&devpriv->timer, jiffies + timer_period());
 }
 
+/* the pc104-das16jr (at least) has problems if the dma
+	transfer is interrupted in the middle of transferring
+	a 16 bit sample, so this function takes care to get
+	an even transfer count after disabling dma
+	channel.
+*/
+static int disable_dma_on_even(comedi_device *dev)
+{
+	int residue;
+	int i;
+	static const int disable_limit = 100;
+	static const int enable_timeout = 100;
+	disable_dma(devpriv->dma_chan);
+	residue = get_dma_residue(devpriv->dma_chan);
+	for(i = 0; i < disable_limit && (residue % 2); ++i)
+	{
+		int j;
+		enable_dma(devpriv->dma_chan);
+		for(j = 0; j < enable_timeout; ++j)
+		{
+			int new_residue;
+			comedi_udelay(2);
+			new_residue = get_dma_residue(devpriv->dma_chan);
+			if(new_residue != residue) break;
+		}
+		disable_dma(devpriv->dma_chan);
+		residue = get_dma_residue(devpriv->dma_chan);
+	}
+	if(i == disable_limit)
+	{
+		comedi_error(dev, "failed to get an even dma transfer, could be trouble.");
+	}
+	return residue;
+}
+
 static void das16_interrupt( comedi_device *dev )
 {
 	unsigned long dma_flags, spin_flags;
@@ -1204,17 +1239,10 @@ static void das16_interrupt( comedi_device *dev )
 	}
 
 	dma_flags = claim_dma_lock();
-	disable_dma(devpriv->dma_chan);
-	/* clear flip-flop to make sure 2-byte registers for
-	 * count and address get set correctly */
 	clear_dma_ff(devpriv->dma_chan);
+	residue = disable_dma_on_even(dev);
 
 	// figure out how many points to read
-
-	/* residue is the number of bytes left to be done on the dma
-	 * transfer.
-	 */
-	residue = get_dma_residue(devpriv->dma_chan);
 	if(residue > devpriv->dma_transfer_size)
 	{
 		comedi_error(dev, "residue > transfer size!\n");
@@ -1393,7 +1421,7 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 
 	if(thisboard->size<0x400){
 		printk(" 0x%04x-0x%04x\n", iobase, iobase+thisboard->size);
-		if(check_region(iobase,thisboard->size)<0){
+		if(!request_region(iobase,thisboard->size,"das16")){
 			printk(" I/O port conflict\n");
 			return -EIO;
 		}
@@ -1401,12 +1429,13 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 		printk(" 0x%04x-0x%04x 0x%04x-0x%04x\n",
 			   iobase,iobase+0x0f,
 			   iobase+0x400,iobase+0x400+(thisboard->size&0x3ff));
-		if(check_region(iobase,0x10) < 0) {
+		if(!request_region(iobase,0x10,"das16")){
 			printk(" I/O port conflict:  0x%04x-0x%04x\n",
 				   iobase,iobase+0x0f);
 			return -EIO;
 		}
-		if(check_region(iobase+0x400,thisboard->size&0x3ff)<0){
+		if(!request_region(iobase+0x400,thisboard->size&0x3ff,"das16")){
+			release_region(iobase, 0x10);
 			printk(" I/O port conflict:  0x%04x-0x%04x\n",
 				   iobase+0x400,
 				   iobase+0x400+(thisboard->size&0x3ff));
@@ -1414,13 +1443,6 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 		}
 	}
 
-	if(thisboard->size < 0x400){
-		request_region(iobase,thisboard->size,"das16");
-	}else{
-		request_region(iobase,0x10,"das16");
-		request_region(iobase+0x400,thisboard->size&0x3ff,"das16");
-	}
-
 	dev->iobase = iobase;
 
 	// probe id bits to make sure they are consistent
