@@ -131,11 +131,10 @@ static int dt2817_attach(comedi_device *dev,comedi_devconfig *it)
 
 	iobase=it->options[0];
 	printk("comedi%d: dt2817: 0x%04x ",dev->minor,iobase);
-	if(check_region(iobase,DT2817_SIZE)<0){
+	if(!request_region(iobase,DT2817_SIZE,"dt2817")){
 		printk("I/O port conflict\n");
 		return -EIO;
 	}
-	request_region(iobase,DT2817_SIZE,"dt2817");
 	dev->iobase = iobase;
 	dev->board_name="dt2817";
 
