@@ -1201,9 +1201,9 @@ enum{	opt_iobase=0, opt_irq, opt_dma1, opt_dma2,	/* i/o base, irq, dma channels
    5	ai 0=straight binary, 1=2's comp
    6	ao0 0=straight binary, 1=2's comp
    7	ao1 0=straight binary, 1=2's comp
-   8	ai 0=10 V, 1=0-10 V, 2=5 V, 3=0-5 V
-   9	ao0 0=10 V, 1=0-10 V, 2=5 V, 3=0-5 V, 4=2.5 V
-   10	ao1 0=10 V, 1=0-10 V, 2=5 V, 3=0-5 V, 4=2.5 V
+   8	ai 0=±10 V, 1=0-10 V, 2=±5 V, 3=0-5 V
+   9	ao0 0=±10 V, 1=0-10 V, 2=±5 V, 3=0-5 V, 4=±2.5 V
+   10	ao1 0=±10 V, 1=0-10 V, 2=±5 V, 3=0-5 V, 4=±2.5 V
  */
 static int dt282x_attach(comedi_device * dev, comedi_devconfig * it)
 {
@@ -1218,11 +1218,10 @@ static int dt282x_attach(comedi_device * dev, comedi_devconfig * it)
 	if(!iobase)iobase = 0x240;
 
 	printk("comedi%d: dt282x: 0x%04x", dev->minor, iobase);
-	if (check_region(iobase, DT2821_SIZE) < 0) {
+	if (!request_region(iobase, DT2821_SIZE, "dt282x")) {
 		printk(" I/O port conflict\n");
 		return -EBUSY;
 	}
-	request_region(iobase, DT2821_SIZE, "dt282x");
 	dev->iobase = iobase;
 
 	outw(DT2821_BDINIT, dev->iobase + DT2821_SUPCSR);
