@@ -309,8 +309,8 @@ typedef struct
 	unsigned long plx9080_phys_iobase;
 	unsigned long hpdi_phys_iobase;
 	// base addresses (ioremapped)
-	unsigned long plx9080_iobase;
-	unsigned long hpdi_iobase;
+	void *plx9080_iobase;
+	void *hpdi_iobase;
 	uint32_t *dio_buffer[ NUM_DMA_BUFFERS ];	// dma buffers
 	dma_addr_t dio_buffer_phys_addr[ NUM_DMA_BUFFERS ];	// physical addresses of dma buffers
 	struct plx_dma_desc *dma_desc;	// array of dma descriptors read by plx9080, allocated to get proper alignment
@@ -377,7 +377,7 @@ static void disable_plx_interrupts( comedi_device *dev )
 static void init_plx9080(comedi_device *dev)
 {
 	uint32_t bits;
-	unsigned long plx_iobase = priv(dev)->plx9080_iobase;
+	void *plx_iobase = priv(dev)->plx9080_iobase;
 
 	// plx9080 dump
 	DEBUG_PRINT(" plx interrupt status 0x%x\n", readl(plx_iobase + PLX_INTRCS_REG));
@@ -561,6 +561,7 @@ static int hpdi_attach(comedi_device *dev, comedi_devconfig *it)
 			}
 			if( pcidev )
 			{
+				priv(dev)->hw_dev = pcidev;
 				dev->board_ptr = hpdi_boards + i;
 				break;
 			}
@@ -577,31 +578,33 @@ static int hpdi_attach(comedi_device *dev, comedi_devconfig *it)
 
 	if( pci_enable_device( pcidev ) )
 	{
-		pci_dev_put( pcidev );
+		printk(KERN_WARNING " failed enable PCI device\n");
 		return -EIO;
 	}
-	pci_set_master( pcidev );
-
-	priv(dev)->hw_dev = pcidev;
-
-	//Initialize dev->board_name
-	dev->board_name = board(dev)->name;
-
 	if( pci_request_regions( pcidev, driver_hpdi.driver_name ) )
 	{
 		/* Couldn't allocate io space */
 		printk(KERN_WARNING " failed to allocate io memory\n");
 		return -EIO;
 	}
+	pci_set_master( pcidev );
+
+	//Initialize dev->board_name
+	dev->board_name = board(dev)->name;
 
 	priv(dev)->plx9080_phys_iobase = pci_resource_start(pcidev, PLX9080_BADDRINDEX);
 	priv(dev)->hpdi_phys_iobase = pci_resource_start(pcidev, HPDI_BADDRINDEX);
 
 	// remap, won't work with 2.0 kernels but who cares
-	priv(dev)->plx9080_iobase = (unsigned long) ioremap( priv(dev)->plx9080_phys_iobase,
+	priv(dev)->plx9080_iobase = ioremap( priv(dev)->plx9080_phys_iobase,
 		pci_resource_len( pcidev, PLX9080_BADDRINDEX ) );
-	priv(dev)->hpdi_iobase = (unsigned long) ioremap( priv(dev)->hpdi_phys_iobase,
+	priv(dev)->hpdi_iobase = ioremap( priv(dev)->hpdi_phys_iobase,
 		pci_resource_len( pcidev, HPDI_BADDRINDEX ) );
+	if (!priv(dev)->plx9080_iobase || !priv(dev)->hpdi_iobase)
+	{
+		printk(" failed to remap io memory\n");
+		return -ENOMEM;
+	}
 
 	DEBUG_PRINT(" plx9080 remapped to 0x%lx\n", priv(dev)->plx9080_iobase);
 	DEBUG_PRINT(" hpdi remapped to 0x%lx\n", priv(dev)->hpdi_iobase);
@@ -667,9 +670,6 @@ static int hpdi_detach(comedi_device *dev)
 			}
 			if( priv(dev)->hpdi_iobase )
 				iounmap((void*)priv(dev)->hpdi_iobase);
-			if( priv(dev)->plx9080_phys_iobase ||
-				priv(dev)->hpdi_phys_iobase )
-				pci_release_regions( priv(dev)->hw_dev );
 			// free pci dma buffers
 			for( i = 0; i < NUM_DMA_BUFFERS; i++ )
 			{
@@ -681,7 +681,11 @@ static int hpdi_detach(comedi_device *dev)
 			if( priv(dev)->dma_desc )
 				pci_free_consistent( priv(dev)->hw_dev, sizeof( struct plx_dma_desc ) *
 				NUM_DMA_DESCRIPTORS, priv(dev)->dma_desc, priv(dev)->dma_desc_phys_addr );
-			pci_disable_device( priv(dev)->hw_dev );
+			if(priv(dev)->hpdi_phys_iobase)
+			{
+				pci_release_regions( priv(dev)->hw_dev );
+				pci_disable_device( priv(dev)->hw_dev );
+			}
 			pci_dev_put(priv(dev)->hw_dev);
 		}
 	}
@@ -877,7 +881,7 @@ static void drain_dma_buffers(comedi_device *dev, unsigned int channel)
 	uint32_t next_transfer_addr;
 	int j;
 	int num_samples = 0;
-	unsigned long pci_addr_reg;
+	void *pci_addr_reg;
 
 	if(channel)
 		pci_addr_reg = priv(dev)->plx9080_iobase + PLX_DMA1_PCI_ADDRESS_REG;
