@@ -268,8 +268,8 @@ COMEDI_INITCLEANUP(me_driver);
 typedef struct
 {
   struct pci_dev* pci_device;
-  unsigned long plx_regbase;         // PLX configuration base address
-  unsigned long me_regbase;          // Base address of the Meilhaus card
+  void *plx_regbase;         // PLX configuration base address
+  void *me_regbase;          // Base address of the Meilhaus card
   unsigned int plx_regbase_size;    // Size of PLX configuration space
   unsigned int me_regbase_size;     // Size of Meilhaus space
 
@@ -688,6 +688,12 @@ static int me_attach(comedi_device *dev,comedi_devconfig *it)
   unsigned int regbase_tmp;
   int result, error, i;
 
+  // Allocate private memory
+  if(alloc_private(dev,sizeof(me_private_data_struct)) < 0)
+  {
+    return -ENOMEM;
+  }
+
 //
 // Probe the device to determine what device in the series it is.
 //
@@ -713,6 +719,7 @@ static int me_attach(comedi_device *dev,comedi_devconfig *it)
 
           dev->board_ptr = me_boards + i;
           board = (me_board_struct *) dev->board_ptr;
+          dev_private->pci_device = pci_device;
           goto found;
         }
       }
@@ -729,28 +736,34 @@ found:
          dev->minor, me_boards[i].name,
          pci_device->bus->number, PCI_SLOT(pci_device->devfn));
 
-  // Allocate private memory
-
-  if(alloc_private(dev,sizeof(me_private_data_struct)) < 0)
+  // Enable PCI device
+  if(pci_enable_device(pci_device) < 0)
   {
-    pci_dev_put(pci_device);
-    return -ENOMEM;
+    printk("comedi%d: Failed to enable PCI device\n", dev->minor);
+    return -EIO;
+  }
+
+  // Request PCI regions
+  if(pci_request_regions(pci_device, ME_DRIVER_NAME) < 0)
+  {
+    printk("comedi%d: I/O memory conflict\n", dev->minor);
+    return -EIO;
   }
 
   // Set data in device structure
 
   dev->board_name = board->name;
-  dev_private->pci_device = pci_device;
 
   // Read PLX register base address [PCI_BASE_ADDRESS #0].
 
   plx_regbase_tmp = pci_resource_start(pci_device, 0);
   plx_regbase_size_tmp = pci_resource_end(pci_device, 0) - plx_regbase_tmp + 1;
-
-  if(plx_regbase_tmp & PCI_BASE_ADDRESS_SPACE)
+  dev_private->plx_regbase = ioremap(plx_regbase_tmp, plx_regbase_size_tmp);
+  dev_private->plx_regbase_size = plx_regbase_size_tmp;
+  if(!dev_private->plx_regbase)
   {
-    printk("comedi%d: PLX space is not MEM\n", dev->minor);
-    return -EIO;
+    printk("comedi%d: Failed to remap I/O memory\n", dev->minor);
+    return -ENOMEM;
   }
 
   // Read Swap base address [PCI_BASE_ADDRESS #5].
@@ -792,24 +805,18 @@ found:
   }
   /*----------------------------------------------------- Workaround end -----*/
 
-  plx_regbase_tmp &= PCI_BASE_ADDRESS_MEM_MASK;
-  dev_private->plx_regbase_size = plx_regbase_size_tmp;
-  dev_private->plx_regbase = (unsigned int) ioremap(plx_regbase_tmp, plx_regbase_size_tmp);
 
   // Read Meilhaus register base address [PCI_BASE_ADDRESS #2].
 
   me_regbase_tmp = pci_resource_start(pci_device, 2);
   me_regbase_size_tmp = pci_resource_end(pci_device, 2) - me_regbase_tmp + 1;
-
-  if(me_regbase_tmp & PCI_BASE_ADDRESS_SPACE)
-  {
-    printk("comedi%d: Meilhaus space is not MEM\n", dev->minor);
-    return -EIO;
-  }
-
-  me_regbase_tmp &= PCI_BASE_ADDRESS_MEM_MASK;
   dev_private->me_regbase_size = me_regbase_size_tmp;
-  dev_private->me_regbase = (unsigned int) ioremap(me_regbase_tmp, me_regbase_size_tmp);
+  dev_private->me_regbase = ioremap(me_regbase_tmp, me_regbase_size_tmp);
+  if(!dev_private->me_regbase)
+  {
+    printk("comedi%d: Failed to remap I/O memory\n", dev->minor);
+    return -ENOMEM;
+  }
 
   // Download firmware and reset card
   if(board->device_id == ME2600_DEVICE_ID)
@@ -820,8 +827,11 @@ found:
     aux_data = comedi_aux_data(it->options, 0);
     aux_len = it->options[COMEDI_DEVCONF_AUX_DATA_LENGTH];
 
-    if(!aux_data || aux_len)return -EINVAL;
-
+		if(!aux_data || aux_len < 1)
+		{
+			comedi_error(dev, "You must provide me2600 firmware using the --init-data option of comedi_config");
+			return -EINVAL;
+		}
     me2600_xilinx_download(dev, aux_data, aux_len);
   }
 
@@ -876,15 +886,24 @@ found:
 
 static int me_detach(comedi_device *dev)
 {
-  if(dev_private)
-  {
-    me_reset(dev);
-
-    if(dev_private->pci_device)
-    {
-      pci_dev_put(dev_private->pci_device);
-    }
-  }
-
-  return 0;
+	if(dev_private)
+	{
+		if (dev_private->me_regbase)
+		{
+			me_reset(dev);
+			iounmap(dev_private->me_regbase);
+		}
+		if (dev_private->plx_regbase)
+			iounmap(dev_private->plx_regbase);
+		if(dev_private->pci_device)
+		{
+			if(dev_private->plx_regbase_size)
+			{
+				pci_release_regions(dev_private->pci_device);
+				pci_disable_device(dev_private->pci_device);
+			}
+			pci_dev_put(dev_private->pci_device);
+		}
+	}
+	return 0;
 }
