@@ -29,7 +29,7 @@ Devices: [National Instruments] AT-MIO-16E-1 (ni_atmio),
 Status: works
 Updated: Thu May  1 20:03:02 CDT 2003
 
-The driver now has (2.4) kernel isapnp support, and
+The driver has 2.6 kernel isapnp support, and
 will automatically probe for a supported board if the
 I/O base is left unspecified with comedi_config.
 However, many of
@@ -128,6 +128,7 @@ static ni_board ni_boards[]={
 		n_aochan:	2,
 		aobits:		12,
 		ao_fifo_depth:	2048,
+		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:	1,
 		has_8255:	0,
 		caldac:		{mb88341},
@@ -144,6 +145,7 @@ static ni_board ni_boards[]={
 		n_aochan:	2,
 		aobits:		12,
 		ao_fifo_depth:	2048,
+		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:	1,
 		has_8255:	0,
 		caldac:		{mb88341},
@@ -160,12 +162,13 @@ static ni_board ni_boards[]={
 		n_aochan:	2,
 		aobits:		12,
 		ao_fifo_depth:	0,
+		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:	1,
 		caldac:		{ad8804_debug},
 		has_8255:	0,
 	},
 	{	device_id:	37,
-		isapnp_id:	0x0000,	/* XXX unknown */
+		isapnp_id:	0x2500,
 		name:		"at-mio-16de-10",
 		n_adchan:	16,
 		adbits:		12,
@@ -176,8 +179,9 @@ static ni_board ni_boards[]={
 		n_aochan:	2,
 		aobits:		12,
 		ao_fifo_depth:	0,
+		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:	1,
-		caldac:		{mb88341},
+		caldac:		{ad8804_debug},
 		has_8255:	1,
 	},
 	{	device_id:	38,
@@ -192,6 +196,7 @@ static ni_board ni_boards[]={
 		n_aochan:	2,
 		aobits:		12,
 		ao_fifo_depth:	2048,
+		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:	1,
 		has_8255:	0,
 		caldac:		{ad8804_debug},
@@ -208,6 +213,7 @@ static ni_board ni_boards[]={
 		n_aochan:	2,
 		aobits:		12,
 		ao_fifo_depth:	0,
+		.ao_range_table = &range_bipolar10,
 		ao_unipolar:	0,
 		caldac:		{dac8800,dac8043},
 		has_8255:	0,
@@ -224,6 +230,7 @@ static ni_board ni_boards[]={
 		n_aochan:	2,
 		aobits:		16,
 		ao_fifo_depth:	2048,
+		.ao_range_table = &range_ni_E_ao_ext,
 		ao_unipolar:	1,
 		caldac:		{dac8800,dac8043,ad8522},
 		has_8255:	0,
@@ -240,7 +247,6 @@ static ni_board ni_boards[]={
 		n_aochan:	0,
 		aobits:		0,
 		ao_fifo_depth:	0,
-		aorangelkup:	0,
 		ao_unipolar:	0,
 		caldac:		{dac8800,dac8043,ad8522},
 		has_8255:	0,
@@ -256,7 +262,7 @@ static int ni_irqpin[]={-1,-1,-1,0,1,2,-1,3,-1,-1,4,5,6,-1,-1,7};
 #define NI_E_IRQ_FLAGS		0
 
 typedef struct{
-	struct pci_dev *isapnp_dev;
+	struct pnp_dev *isapnp_dev;
 	NI_PRIVATE_COMMON
 }ni_private;
 #define devpriv ((ni_private *)dev->private)
@@ -276,8 +282,7 @@ typedef struct{
  * read/written directly in the I/O space of the board.  The
  * AT-MIO devices map the low 8 STC registers to iobase+addr*2. */
 
-#define win_out(data,addr) __win_out(dev,data,addr)
-static inline void __win_out(comedi_device *dev, unsigned short data, int addr)
+static void ni_atmio_win_out(comedi_device *dev, uint16_t data, int addr)
 {
 	unsigned long flags;
 
@@ -291,11 +296,10 @@ static inline void __win_out(comedi_device *dev, unsigned short data, int addr)
 	comedi_spin_unlock_irqrestore(&devpriv->window_lock,flags);
 }
 
-#define win_in(addr) __win_in(dev,addr)
-static inline unsigned short __win_in(comedi_device *dev, int addr)
+static uint16_t ni_atmio_win_in(comedi_device *dev, int addr)
 {
 	unsigned long flags;
-	int ret;
+	uint16_t ret;
 
 	comedi_spin_lock_irqsave(&devpriv->window_lock,flags);
 	if(addr<8){
@@ -309,15 +313,15 @@ static inline unsigned short __win_in(comedi_device *dev, int addr)
 	return ret;
 }
 
-#ifdef __ISAPNP__
-static struct isapnp_device_id device_ids[] = {
-	{ ISAPNP_DEVICE_SINGLE('N','I','C',0x1900,'N','I','C',0x0000), },
-	{ ISAPNP_DEVICE_SINGLE('N','I','C',0x2400,'N','I','C',0x2400), },
-	{ ISAPNP_DEVICE_SINGLE('N','I','C',0x2700,'N','I','C',0x2700), },
-	{ ISAPNP_DEVICE_SINGLE_END, },
+static struct pnp_device_id device_ids[] = {
+	{.id = "NIC1900", .driver_data = 0},
+	{.id = "NIC2400", .driver_data = 0},
+	{.id = "NIC2500", .driver_data = 0},
+	{.id = "NIC2600", .driver_data = 0},
+	{.id = "NIC2700", .driver_data = 0},
+	{.id = ""}
 };
-MODULE_DEVICE_TABLE(isapnp, device_ids);
-#endif
+MODULE_DEVICE_TABLE(pnp, device_ids);
 
 static int ni_atmio_attach(comedi_device *dev,comedi_devconfig *it);
 static int ni_atmio_detach(comedi_device *dev);
@@ -344,51 +348,40 @@ static int ni_atmio_detach(comedi_device *dev)
 	if(dev->irq){
 		comedi_free_irq(dev->irq,dev);
 	}
-#ifdef __ISAPNP__
 	if(devpriv->isapnp_dev)
-		devpriv->isapnp_dev->deactivate(devpriv->isapnp_dev);
-#endif
+		pnp_device_detach(devpriv->isapnp_dev);
 
 	return 0;
 }
 
-static int ni_isapnp_find_board( struct pci_dev **dev )
+static int ni_isapnp_find_board( struct pnp_dev **dev )
 {
-#ifdef __ISAPNP__
-	struct pci_dev *isapnp_dev = NULL;
+	struct pnp_dev *isapnp_dev = NULL;
 	int i;
 	
 	for( i = 0; i < n_ni_boards; i++ )
 	{
-		isapnp_dev = isapnp_find_dev(NULL,
+		isapnp_dev = pnp_find_dev(NULL,
 			ISAPNP_VENDOR('N','I','C'),
 			ISAPNP_FUNCTION( ni_boards[ i ].isapnp_id ),
 			NULL);
 
-		if(!isapnp_dev) continue;
+		if(isapnp_dev == NULL || isapnp_dev->card == NULL) continue;
 
-		if(isapnp_dev->active)
+		if(pnp_device_attach(isapnp_dev) < 0)
 		{
 			printk( "ni_atmio: %s found but already active, skipping.\n", ni_boards[ i ].name );
 			continue;
 		}
-		if(isapnp_dev->prepare(isapnp_dev)<0)
+		if(pnp_activate_dev(isapnp_dev) < 0)
+		{
+			pnp_device_detach(isapnp_dev);
 			return -EAGAIN;
-
-		if(!(isapnp_dev->resource[0].flags & IORESOURCE_IO))
-			return -ENODEV;
-
-#if 0
-		if(!isapnp_dev->ro){
-			/* override resource */
-			if(it->options[0] != 0){
-				isapnp_resource_change(&isapnp_dev->resource[0],
-					it->options[0], NI_SIZE);
-			}
 		}
-#endif
-		if(isapnp_dev->activate(isapnp_dev)<0){
-			printk("ni_atmio: isapnp configure failed!\n");
+		if(!pnp_port_valid(isapnp_dev, 0) || !pnp_irq_valid(isapnp_dev, 0))
+		{
+			pnp_device_detach(isapnp_dev);
+			printk("ni_atmio: pnp invalid port or irq, aborting\n");
 			return -ENOMEM;
 		}
 		break;
@@ -396,15 +389,11 @@ static int ni_isapnp_find_board( struct pci_dev **dev )
 	if( i == n_ni_boards ) return -ENODEV;
 	*dev = isapnp_dev;
 	return 0;
-#else
-	printk("ni_atmio: kernel does not have isapnp support available\n");
-	return -EIO;
-#endif
 }
 
 static int ni_atmio_attach(comedi_device *dev,comedi_devconfig *it)
 {
-	struct pci_dev *isapnp_dev;
+	struct pnp_dev *isapnp_dev;
 	int		ret;
 	int		iobase;
 	int		board;
@@ -413,6 +402,9 @@ static int ni_atmio_attach(comedi_device *dev,comedi_devconfig *it)
 	/* allocate private area */
 	if((ret = ni_alloc_private(dev)) < 0)
 		return ret;
+	devpriv->stc_writew = &ni_atmio_win_out;
+	devpriv->stc_readw = &ni_atmio_win_in;
+	devpriv->stc_writel = &win_out2;
 
 	iobase=it->options[0];
 	irq=it->options[1];
@@ -422,23 +414,18 @@ static int ni_atmio_attach(comedi_device *dev,comedi_devconfig *it)
 		ret = ni_isapnp_find_board( &isapnp_dev );
 		if( ret < 0 ) return ret;
 
-#ifdef __ISAPNP__
-		iobase = isapnp_dev->resource[0].start;
-		irq = isapnp_dev->irq_resource[0].start;
+		iobase = pnp_port_start(isapnp_dev, 0);
+		irq = pnp_irq(isapnp_dev, 0);
 		devpriv->isapnp_dev = isapnp_dev;
-#else
-		return -EIO;
-#endif
 	}
 
 	/* reserve our I/O region */
 
 	printk("comedi%d: ni_atmio: 0x%04x",dev->minor,iobase);
-	if(check_region(iobase,NI_SIZE)<0){
+	if(!request_region(iobase,NI_SIZE,"ni_atmio")){
 		printk(" I/O port conflict\n");
 		return -EIO;
 	}
-	request_region(iobase,NI_SIZE,"ni_atmio");
 
 	dev->iobase=iobase;
 
