@@ -43,10 +43,10 @@ the PCMCIA interface.
 #include <linux/comedidev.h>
 
 #include <linux/ioport.h>
+#include <linux/version.h>
 
 #include "8255.h"
 
-#include <pcmcia/version.h>
 #include <pcmcia/cs_types.h>
 #include <pcmcia/cs.h>
 #include <pcmcia/cistpl.h>
@@ -91,6 +91,12 @@ static dio24_board dio24_boards[] =
 		bustype:	pcmcia_bustype,
 		have_dio:	1,
 	},
+	{
+		name:	"ni_daq_dio24",
+		device_id:	0x475c,	// 0x10b is manufacturer id, 0x475c is device id
+		bustype:	pcmcia_bustype,
+		have_dio:	1,
+	},
 };
 
 /*
@@ -204,7 +210,7 @@ static int dio24_detach(comedi_device *dev)
 */
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
-MODULE_PARM(pc_debug, "i");
+module_param(pc_debug, int, 0644);
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
 "ni_daq_dio24.c, based on dummy_cs.c";
@@ -214,32 +220,10 @@ static char *version =
 
 /*====================================================================*/
 
-/* Parameters that can be set with 'insmod' */
-
-/* The old way: bit map of interrupts to choose from */
-/* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
-static u_int irq_mask = 0xdeb8;
-/* Newer, simpler way of listing specific interrupts */
-static int irq_list[4] = { -1 };
-
-MODULE_PARM(irq_mask, "i");
-MODULE_PARM(irq_list, "1-4i");
-
-/*====================================================================*/
-
-/*
-   The event() function is this driver's Card Services event handler.
-   It will be called by Card Services when an appropriate card status
-   event is received.  The config() and release() entry points are
-   used to configure or release a socket, in response to card
-   insertion and ejection events.  They are invoked from the dummy
-   event handler.
-*/
-
 static void dio24_config(dev_link_t *link);
 static void dio24_release(u_long arg);
-static int dio24_event(event_t event, int priority,
-		       event_callback_args_t *args);
+static int dio24_cs_suspend(struct pcmcia_device *p_dev);
+static int dio24_cs_resume(struct pcmcia_device *p_dev);
 
 /*
    The attach() and detach() entry points are used to create and destroy
@@ -247,8 +231,8 @@ static int dio24_event(event_t event, int priority,
    needed to manage one actual PCMCIA card.
 */
 
-static dev_link_t *dio24_cs_attach(void);
-static void dio24_cs_detach(dev_link_t *);
+static int dio24_cs_attach(struct pcmcia_device *);
+static void dio24_cs_detach(struct pcmcia_device *);
 
 /*
    You'll also need to prototype all the functions that will actually
@@ -313,12 +297,10 @@ typedef struct local_info_t {
 
 ======================================================================*/
 
-static dev_link_t *dio24_cs_attach(void)
+static int dio24_cs_attach(struct pcmcia_device *p_dev)
 {
     local_info_t *local;
     dev_link_t *link;
-    client_reg_t client_reg;
-    int ret, i;
 
     printk(KERN_INFO "ni_daq_dio24: HOLA SOY YO - CS-attach!\n");
 
@@ -326,23 +308,13 @@ static dev_link_t *dio24_cs_attach(void)
 
     /* Allocate space for private device-specific data */
     local = kmalloc(sizeof(local_info_t), GFP_KERNEL);
-    if (!local) return NULL;
+    if (!local) return -ENOMEM;
     memset(local, 0, sizeof(local_info_t));
     link = &local->link; link->priv = local;
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-    /* Initialize the dev_link_t structure */
-    link->release.function = &dio24_release;
-    link->release.data = (u_long)link;
-#endif
     /* Interrupt setup */
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
-    link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-    if (irq_list[0] == -1)
-	link->irq.IRQInfo2 = irq_mask;
-    else
-	for (i = 0; i < 4; i++)
-	    link->irq.IRQInfo2 |= 1 << irq_list[i];
+    link->irq.IRQInfo1 = IRQ_LEVEL_ID;
     link->irq.Handler = NULL;
 
     /*
@@ -356,26 +328,15 @@ static dev_link_t *dio24_cs_attach(void)
     link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
 
-    /* Register with Card Services */
     link->next = pcmcia_dev_list;
     pcmcia_dev_list = link;
-    client_reg.dev_info = &dev_info;
-    client_reg.Attributes = INFO_IO_CLIENT | INFO_CARD_SHARE;
-    client_reg.EventMask =
-	CS_EVENT_CARD_INSERTION | CS_EVENT_CARD_REMOVAL |
-	CS_EVENT_RESET_PHYSICAL | CS_EVENT_CARD_RESET |
-	CS_EVENT_PM_SUSPEND | CS_EVENT_PM_RESUME;
-    client_reg.event_handler = &dio24_event;
-    client_reg.Version = 0x0210;
-    client_reg.event_callback_args.client_data = link;
-    ret = pcmcia_register_client(&link->handle, &client_reg);
-    if (ret != CS_SUCCESS) {
-	cs_error(link->handle, RegisterClient, ret);
-	dio24_cs_detach(link);
-	return NULL;
-    }
 
-    return link;
+    link->handle = p_dev;
+    p_dev->instance = link;
+    link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
+    dio24_config(link);
+
+    return 0;
 } /* dio24_cs_attach */
 
 /*======================================================================
@@ -387,8 +348,9 @@ static dev_link_t *dio24_cs_attach(void)
 
 ======================================================================*/
 
-static void dio24_cs_detach(dev_link_t *link)
+static void dio24_cs_detach(struct pcmcia_device *p_dev)
 {
+    dev_link_t *link = dev_to_instance(p_dev);
     dev_link_t **linkp;
 
     printk(KERN_INFO "ni_daq_dio24: HOLA SOY YO - cs-detach!\n");
@@ -408,18 +370,10 @@ static void dio24_cs_detach(dev_link_t *link)
        detach().
     */
     if (link->state & DEV_CONFIG) {
-#ifdef PCMCIA_DEBUG
-	printk(KERN_DEBUG "ni_daq_dio24: detach postponed, '%s' "
-	       "still locked\n", link->dev->dev_name);
-#endif
-	link->state |= DEV_STALE_LINK;
-	return;
+	((local_info_t *)link->priv)->stop = 1;
+	dio24_release((u_long)link);
     }
 
-    /* Break the link with Card Services */
-    if (link->handle)
-		pcmcia_deregister_client(link->handle);
-
     /* Unlink device structure, and free it */
     *linkp = link->next;
     /* This points to the parent local_info_t struct */
@@ -677,18 +631,6 @@ static void dio24_release(u_long arg)
 
     DEBUG(0, "dio24_release(0x%p)\n", link);
 
-    /*
-       If the device is currently in use, we won't release until it
-       is actually closed, because until then, we can't be sure that
-       no one will try to access the device or its data structures.
-    */
-    if (link->open) {
-	DEBUG(1, "ni_dio24: release postponed, '%s' still open\n",
-	      link->dev->dev_name);
-	link->state |= DEV_STALE_CONFIG;
-	return;
-    }
-
     /* Unlink the device chain */
     link->dev = NULL;
 
@@ -707,9 +649,6 @@ static void dio24_release(u_long arg)
 		pcmcia_release_irq(link->handle, &link->irq);
     link->state &= ~DEV_CONFIG;
 
-    if (link->state & DEV_STALE_LINK)
-	dio24_cs_detach(link);
-
 } /* dio24_release */
 
 /*======================================================================
@@ -724,79 +663,61 @@ static void dio24_release(u_long arg)
 
 ======================================================================*/
 
-static int dio24_event(event_t event, int priority,
-		       event_callback_args_t *args)
+
+static int dio24_cs_suspend(struct pcmcia_device *p_dev)
 {
-    dev_link_t *link = args->client_data;
-    local_info_t *dev = link->priv;
+	dev_link_t *link = dev_to_instance(p_dev);
+	local_info_t *local = link->priv;
 
-    DEBUG(1, "dio24_event(0x%06x)\n", event);
-
-    switch (event) {
-    case CS_EVENT_CARD_REMOVAL:
-	link->state &= ~DEV_PRESENT;
-	if (link->state & DEV_CONFIG) {
-		((local_info_t *)link->priv)->stop = 1;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-		mod_timer(&link->release, jiffies + HZ/20);
-#else	    
-		dio24_release((u_long)pcmcia_dev_list);
-#endif
-	}
-	break;
-    case CS_EVENT_CARD_INSERTION:
-	link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
-//	dev->bus = args->bus;
-	dio24_config(link);
-	break;
-    case CS_EVENT_PM_SUSPEND:
 	link->state |= DEV_SUSPEND;
-	/* Fall through... */
-    case CS_EVENT_RESET_PHYSICAL:
 	/* Mark the device as stopped, to block IO until later */
-	dev->stop = 1;
+	local->stop = 1;
 	if (link->state & DEV_CONFIG)
-	    pcmcia_release_configuration(link->handle);
-	break;
-    case CS_EVENT_PM_RESUME:
+		pcmcia_release_configuration(link->handle);
+
+	return 0;
+} /* dio24_cs_suspend */
+
+static int dio24_cs_resume(struct pcmcia_device *p_dev)
+{
+	dev_link_t *link = dev_to_instance(p_dev);
+	local_info_t *local = link->priv;
+
 	link->state &= ~DEV_SUSPEND;
-	/* Fall through... */
-    case CS_EVENT_CARD_RESET:
 	if (link->state & DEV_CONFIG)
 		pcmcia_request_configuration(link->handle, &link->conf);
-	dev->stop = 0;
-	/*
-	  In a normal driver, additional code may go here to restore
-	  the device state and restart IO.
-	*/
-	break;
-    }
-    return 0;
-} /* dio24_event */
+	local->stop = 0;
+	return 0;
+} /* dio24_cs_resume */
 
 /*====================================================================*/
 
+static struct pcmcia_device_id dio24_cs_ids[] =
+{
+	/* N.B. These IDs should match those in dio24_boards */
+	PCMCIA_DEVICE_MANF_CARD(0x010b, 0x475c),	/* daqcard-dio24 */
+	PCMCIA_DEVICE_NULL
+};
+
+MODULE_DEVICE_TABLE(pcmcia, dio24_cs_ids);
+
 struct pcmcia_driver dio24_cs_driver =
 {
-	.attach = dio24_cs_attach,
-	.detach = dio24_cs_detach,
+	.probe = dio24_cs_attach,
+	.remove = dio24_cs_detach,
+	.suspend = dio24_cs_suspend,
+	.resume = dio24_cs_resume,
+	.id_table = dio24_cs_ids,
 	.owner = THIS_MODULE,
 	.drv = {
-		.name = "ni_daq_dio24",
+		.name = dev_info,
 	},	
 };
 
 static int __init init_dio24_cs(void)
 {
-    servinfo_t serv;
     printk("ni_daq_dio24: HOLA SOY YO!\n");
     DEBUG(0, "%s\n", version);
-	pcmcia_get_card_services_info(&serv);
-    if (serv.Revision != CS_RELEASE_CODE) {
-		printk(KERN_NOTICE "ni_daq_dio24: Card Services release "
-			"does not match! Vaya putada\n");
-		return -1;
-    }
 	pcmcia_register_driver(&dio24_cs_driver);
     return 0;
 }
@@ -806,16 +727,13 @@ static void __exit exit_dio24_cs(void)
     DEBUG(0, "ni_dio24: unloading\n");
 	pcmcia_unregister_driver(&dio24_cs_driver);
     while (pcmcia_dev_list != NULL) {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-		del_timer(&pcmcia_dev_list->release);
-#endif
 		if (pcmcia_dev_list->state & DEV_CONFIG)
 			dio24_release((u_long)pcmcia_dev_list);
-		dio24_cs_detach(pcmcia_dev_list);
+		dio24_cs_detach(pcmcia_dev_list->handle);
     }
 }
 
-int init_module(void)
+int __init init_module(void)
 {
 	int ret;
 
@@ -826,7 +744,7 @@ int init_module(void)
 	return comedi_driver_register(&driver_dio24);
 }
 
-void cleanup_module(void)
+void __exit cleanup_module(void)
 {
 	exit_dio24_cs();
 	comedi_driver_unregister(&driver_dio24);
