@@ -44,11 +44,11 @@ See the notes in the ni_atmio.o driver.
 #include <linux/comedidev.h>
 
 #include <linux/delay.h>
+#include <linux/version.h>
 
 #include "ni_stc.h"
 #include "8255.h"
 
-#include <pcmcia/version.h>
 #include <pcmcia/cs_types.h>
 #include <pcmcia/cs.h>
 #include <pcmcia/cistpl.h>
@@ -109,6 +109,7 @@ static ni_board ni_boards[]={
 		n_aochan:	2,
 		aobits:		12,
 		ao_fifo_depth:	2048,
+		.ao_range_table = &range_bipolar10,
 		ao_unipolar:	0,
 		has_8255:	0,
 		caldac:		{ad8804_debug},	/* verified */
@@ -124,6 +125,7 @@ static ni_board ni_boards[]={
 		n_aochan:	2,
 		aobits:		12,
 		ao_fifo_depth:	0,
+		.ao_range_table = &range_bipolar10,
 		ao_unipolar:	0,
 		has_8255:	0,
 		caldac:		{ad8804_debug},
@@ -139,6 +141,7 @@ static ni_board ni_boards[]={
 		n_aochan:	2,
 		aobits:		16,
 		ao_fifo_depth:	0,
+		.ao_range_table = &range_bipolar10,
 		ao_unipolar:	0,
 		has_8255:	0,
 		caldac:		{ad8804_debug},
@@ -153,6 +156,7 @@ static ni_board ni_boards[]={
 		caldac:		{mb88341,mb88341},
 	},
 #endif
+	/* N.B. Update ni_mio_cs_ids[] when entries added above. */
 };
 
 
@@ -184,8 +188,7 @@ typedef struct{
  * read/written directly in the I/O space of the board.  The
  * DAQCard devices map the low 8 STC registers to iobase+addr*2. */
 
-#define win_out(data,addr) __win_out(dev,data,addr)
-static inline void __win_out(comedi_device *dev, unsigned short data, int addr)
+static void mio_cs_win_out(comedi_device *dev, uint16_t data, int addr)
 {
 	unsigned long flags;
 
@@ -199,11 +202,10 @@ static inline void __win_out(comedi_device *dev, unsigned short data, int addr)
 	comedi_spin_unlock_irqrestore(&devpriv->window_lock,flags);
 }
 
-#define win_in(addr) __win_in(dev,addr)
-static inline unsigned short __win_in(comedi_device *dev, int addr)
+static uint16_t mio_cs_win_in(comedi_device *dev, int addr)
 {
 	unsigned long flags;
-	int ret;
+	uint16_t ret;
 
 	comedi_spin_lock_irqsave(&devpriv->window_lock,flags);
 	if(addr<8){
@@ -249,8 +251,7 @@ static int mio_cs_detach(comedi_device *dev)
 
 static void mio_cs_config(dev_link_t *link);
 static void cs_release(u_long arg);
-static void cs_detach(dev_link_t *);
-static int irq_mask;
+static void cs_detach(struct pcmcia_device *);
 
 static dev_link_t *dev_list = NULL;
 static dev_info_t dev_info = "ni_mio_cs";
@@ -259,27 +260,18 @@ static dev_node_t dev_node = {
 	COMEDI_MAJOR,0,
 	NULL
 };
-static int mio_cs_event(event_t event, int priority, event_callback_args_t *args);
-
-static dev_link_t *cs_attach(void)
+static int cs_attach(struct pcmcia_device *p_dev)
 {
 	dev_link_t *link;
-	client_reg_t client_reg;
-	int ret;
 
 	link=kmalloc(sizeof(*link),GFP_KERNEL);
-	if(!link)return NULL;
+	if(!link)return -ENOMEM;
 	memset(link,0,sizeof(*link));
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-	link->release.function = &cs_release;
-	link->release.data = (u_long)link;
-#endif	
 	link->io.Attributes1 = IO_DATA_PATH_WIDTH_16;
 	link->io.NumPorts1 = 16;
 	link->irq.Attributes = IRQ_TYPE_EXCLUSIVE;
-	link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-	link->irq.IRQInfo2 = irq_mask;
+	link->irq.IRQInfo1 = IRQ_LEVEL_ID;
 	link->conf.Attributes = CONF_ENABLE_IRQ;
 	link->conf.Vcc = 50;
 	link->conf.IntType = INT_MEMORY_AND_IO;
@@ -287,24 +279,12 @@ static dev_link_t *cs_attach(void)
 	link->next = dev_list;
 	dev_list = link;
 
-	client_reg.dev_info = &dev_info;
-	client_reg.Attributes = INFO_IO_CLIENT | INFO_CARD_SHARE;
-	client_reg.EventMask =
-		CS_EVENT_CARD_INSERTION | CS_EVENT_CARD_REMOVAL |
-		CS_EVENT_RESET_PHYSICAL | CS_EVENT_CARD_RESET |
-		CS_EVENT_PM_SUSPEND | CS_EVENT_PM_RESUME;
-	client_reg.event_handler = &mio_cs_event;
-	client_reg.Version = 0x0210;
-	client_reg.event_callback_args.client_data = link;
-	ret = pcmcia_register_client(&link->handle, &client_reg);
-	if (ret != CS_SUCCESS) {
-		cs_error(link->handle, RegisterClient, ret);
-		printk("detaching...\n");
-		cs_detach(link);
-		return NULL;
-	}
+	link->handle = p_dev;
+	p_dev->instance = link;
+	link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
+	mio_cs_config(link);
 
-	return link;
+	return 0;
 }
 
 static void cs_release(u_long arg)
@@ -318,8 +298,9 @@ static void cs_release(u_long arg)
 	link->state &= ~DEV_CONFIG;
 }
 
-static void cs_detach(dev_link_t *link)
+static void cs_detach(struct pcmcia_device *p_dev)
 {
+	dev_link_t *link = dev_to_instance(p_dev);
 	dev_link_t **linkp;
 	
 	DPRINTK("cs_detach(link=%p)\n",link);
@@ -333,78 +314,42 @@ static void cs_detach(dev_link_t *link)
 	//cli
 	if (link->state & DEV_RELEASE_PENDING){
 		printk("dev release pending bug\n");
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-		del_timer(&link->release);
-#endif
 		link->state &= ~DEV_RELEASE_PENDING;
 	}
 	//restore_flags
 
 	if(link->state & DEV_CONFIG) {
 		cs_release((u_long)link);
-		if(link->state & DEV_STALE_CONFIG) {
-			link->state |= DEV_STALE_LINK;
-			return;
-		}
-	}
-
-	if(link->handle){
-		pcmcia_deregister_client(link->handle);
 	}
 
+	/* Unlink device structure, and free it */
+	*linkp = link->next;
+	kfree(link);
 }
 
-static int mio_cs_event(event_t event, int priority, event_callback_args_t *args)
+static int mio_cs_suspend(struct pcmcia_device *p_dev)
 {
-	dev_link_t *link = args->client_data;
+	dev_link_t *link = dev_to_instance(p_dev);
 
-	DPRINTK("mio_cs_event(event=%x,priority=%d,args=%p)\n",event,priority,args);
+	DPRINTK("pm suspend\n");
+	link->state |= DEV_SUSPEND;
+	if(link->state & DEV_CONFIG)
+		pcmcia_release_configuration(link->handle);
 
-	switch(event){
-	case CS_EVENT_CARD_REMOVAL:
-		DPRINTK("removal event\n");
-		link->state &= ~DEV_PRESENT;
-		if(link->state & DEV_CONFIG) {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-			link->release.expires = jiffies+HZ/20;
-			link->state |= DEV_RELEASE_PENDING;
-			add_timer(&link->release);
-#else
-			cs_release((ulong)link);	
-#endif
-		}
-		/* XXX disable irq here, to get rid of spurious interrupts */
-		break;
-	case CS_EVENT_CARD_INSERTION:
-		DPRINTK("card insertion event\n");
-		link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
-		mio_cs_config(link);
-		break;
-	case CS_EVENT_PM_SUSPEND:
-		DPRINTK("pm suspend event\n");
-		link->state |= DEV_SUSPEND;
-		/* fall through */
-	case CS_EVENT_RESET_PHYSICAL:
-		DPRINTK("reset physical event\n");
-		if(link->state & DEV_CONFIG)
-			pcmcia_release_configuration(link->handle);
-		break;
-	case CS_EVENT_PM_RESUME:
-		DPRINTK("pm resume event\n");
-		link->state &= ~DEV_SUSPEND;
-		/* fall through */
-	case CS_EVENT_CARD_RESET:
-		DPRINTK("card reset event\n");
-		if(DEV_OK(link))
-			pcmcia_request_configuration(link->handle, &link->conf);
-		break;
-	default:
-		DPRINTK("unknown event (ignored)\n");
-	}
 	return 0;
 }
 
+static int mio_cs_resume(struct pcmcia_device *p_dev)
+{
+	dev_link_t *link = dev_to_instance(p_dev);
 
+	DPRINTK("pm resume\n");
+	link->state &= ~DEV_SUSPEND;
+	if(DEV_OK(link))
+		pcmcia_request_configuration(link->handle, &link->conf);
+
+	return 0;
+}
 
 static void mio_cs_config(dev_link_t *link)
 {
@@ -552,6 +497,9 @@ static int mio_cs_attach(comedi_device *dev,comedi_devconfig *it)
 	/* allocate private area */
 	if((ret=ni_alloc_private(dev))<0)
 		return ret;
+	devpriv->stc_writew = &mio_cs_win_out;
+	devpriv->stc_readw = &mio_cs_win_in;
+	devpriv->stc_writel = &win_out2;
 	
 	if( (ret=ni_E_init(dev,it))<0 ){
 		return ret;
@@ -604,26 +552,33 @@ static int ni_getboardtype(comedi_device *dev,dev_link_t *link)
 
 MODULE_LICENSE("GPL");
 
+static struct pcmcia_device_id ni_mio_cs_ids[] =
+{
+	PCMCIA_DEVICE_MANF_CARD(0x010b, 0x010d), /* DAQCard-ai-16xe-50 */
+	PCMCIA_DEVICE_MANF_CARD(0x010b, 0x010c), /* DAQCard-ai-16e-4 */
+	PCMCIA_DEVICE_MANF_CARD(0x010b, 0x02c4), /* DAQCard-6062E */
+	PCMCIA_DEVICE_MANF_CARD(0x010b, 0x075e), /* DAQCard-6024E */
+	PCMCIA_DEVICE_MANF_CARD(0x010b, 0x0245), /* DAQCard-6036E */
+	PCMCIA_DEVICE_NULL
+};
+
+MODULE_DEVICE_TABLE(pcmcia, ni_mio_cs_ids);
+
 struct pcmcia_driver ni_mio_cs_driver =
 {
-	.attach = cs_attach,
-	.detach = cs_detach,
+	.probe = &cs_attach,
+	.remove = &cs_detach,
+	.suspend = &mio_cs_suspend,
+	.resume = &mio_cs_resume,
+	.id_table = ni_mio_cs_ids,
 	.owner = THIS_MODULE,
 	.drv = {
-		.name = "ni_mio_cs",
+		.name = dev_info,
 	},	
 };
 
 int init_module(void)
 {
-	servinfo_t serv;
-
-	pcmcia_get_card_services_info(&serv);
-	if(serv.Revision != CS_RELEASE_CODE){
-		printk(KERN_NOTICE "mio_cs: Card Services release "
-			"does not match!\n");
-		//return -EPERM; /* XXX what to return? */
-	}
 	pcmcia_register_driver(&ni_mio_cs_driver);
 	comedi_driver_register(&driver_ni_mio_cs);
 	return 0;
@@ -634,7 +589,7 @@ void cleanup_module(void)
 	pcmcia_unregister_driver(&ni_mio_cs_driver);
 #if 0
 	while(dev_list != NULL)
-		cs_detach(dev_list);
+		cs_detach(dev_list->handle);
 #endif
 	comedi_driver_unregister(&driver_ni_mio_cs);
 }
