@@ -122,7 +122,7 @@ enum Interrupt_B_Ack_Bits
 #define AI_START1_Pulse				_bit0
 
 #define AO_Command_2_Register		5
-#define AO_End_On_BC_TC(x)			((x)<<14)
+#define AO_End_On_BC_TC(x)			(((x) & 0x3) << 14)
 #define AO_Start_Stop_Gate_Enable		_bit13
 #define AO_UC_Save_Trace			_bit12
 #define AO_BC_Gate_Enable			_bit11
@@ -195,9 +195,10 @@ enum Interrupt_B_Ack_Bits
 #define DIO_Pins_Dir_Mask                       0xff
 
 #define AI_Mode_1_Register		12
-#define AI_CONVERT_Source_Select(a)		((a)<<11)
-#define AI_SI_Source_select(a)			((a)<<6)
+#define AI_CONVERT_Source_Select(a)		(((a) & 0x1f) << 11)
+#define AI_SI_Source_select(a)			(((a) & 0x1f) << 6)
 #define AI_CONVERT_Source_Polarity		_bit5
+#define AI_SI_Source_Polarity		_bit4
 #define AI_Start_Stop				_bit3
 #define AI_Mode_1_Reserved			_bit2
 #define AI_Continuous				_bit1
@@ -211,7 +212,7 @@ enum Interrupt_B_Ack_Bits
 #define AI_SI2_Initial_Load_Source		_bit9
 #define AI_SI2_Reload_Mode			_bit8
 #define AI_SI_Initial_Load_Source		_bit7
-#define AI_SI_Reload_Mode(a)			((a)<<4)
+#define AI_SI_Reload_Mode(a)			(((a) & 0x7)<<4)
 #define AI_SI_Write_Switch			_bit3
 #define AI_SC_Initial_Load_Source		_bit2
 #define AI_SC_Reload_Mode			_bit1
@@ -221,6 +222,9 @@ enum Interrupt_B_Ack_Bits
 #define AI_SI_Load_B_Registers		16
 #define AI_SC_Load_A_Registers		18
 #define AI_SC_Load_B_Registers		20
+#define AI_SI_Save_Registers		64
+#define AI_SC_Save_Registers		66
+
 #define AI_SI2_Load_A_Register		23
 #define AI_SI2_Load_B_Register		25
 
@@ -260,7 +264,7 @@ enum AO_FIFO_Mode_Bits
 #define AO_UI2_Initial_Load_Source		_bit9
 #define AO_UI2_Reload_Mode			_bit8
 #define AO_UI_Initial_Load_Source		_bit7
-#define AO_UI_Reload_Mode(x)			((x)<<4)
+#define AO_UI_Reload_Mode(x)			(((x) & 0x7) << 4)
 #define AO_UI_Write_Switch			_bit3
 #define AO_BC_Initial_Load_Source		_bit2
 #define AO_BC_Reload_Mode			_bit1
@@ -269,15 +273,20 @@ enum AO_FIFO_Mode_Bits
 #define AO_UI_Load_A_Register		40
 #define AO_UI_Load_A_Register_High	40
 #define AO_UI_Load_A_Register_Low	41
+#define AO_UI_Load_B_Register		42
+#define AO_UI_Save_Registers		16
 #define AO_BC_Load_A_Register		44
 #define AO_BC_Load_A_Register_High	44
 #define AO_BC_Load_A_Register_Low	45
 #define AO_BC_Load_B_Register		46
 #define AO_BC_Load_B_Register_High	46
 #define AO_BC_Load_B_Register_Low	47
+#define AO_BC_Save_Registers		18
 #define AO_UC_Load_A_Register		48
 #define AO_UC_Load_A_Register_High	48
 #define AO_UC_Load_A_Register_Low	49
+#define AO_UC_Load_B_Register		50
+#define AO_UC_Save_Registers		20
 
 #define Clock_and_FOUT_Register		56
 #define FOUT_Enable				_bit15
@@ -292,9 +301,10 @@ enum AO_FIFO_Mode_Bits
 #define AI_Source_Divide_By_2			_bit6
 #define AO_Output_Divide_By_2			_bit5
 #define AO_Source_Divide_By_2			_bit4
-#define FOUT_Divider(x)				((x)<<0)
+#define FOUT_Divider(x)				(((x) & 0xf) << 0)
 
 #define IO_Bidirection_Pin_Register	57
+#define	RTSI_Trig_Direction_Register	58
 
 #define Interrupt_Control_Register	59
 #define Interrupt_B_Enable			_bit15
@@ -308,31 +318,31 @@ enum AO_FIFO_Mode_Bits
 
 #define AI_Output_Control_Register	60
 #define AI_START_Output_Select			_bit10
-#define AI_SCAN_IN_PROG_Output_Select(x)	((x)<<8)
-#define AI_EXTMUX_CLK_Output_Select(x)		((x)<<6)
+#define AI_SCAN_IN_PROG_Output_Select(x)	(((x) & 0x3) << 8)
+#define AI_EXTMUX_CLK_Output_Select(x)		(((x) & 0x3) << 6)
 #define AI_LOCALMUX_CLK_Output_Select(x)	((x)<<4)
 #define AI_SC_TC_Output_Select(x)		((x)<<2)
-#define AI_CONVERT_Output_Select(x)		((x)<<0)
+#define AI_CONVERT_Output_Select(x)		(((x) & 0x3) << 0)
 
 #define AI_START_STOP_Select_Register	62
 #define AI_START_Polarity			_bit15
 #define AI_STOP_Polarity			_bit14
 #define AI_STOP_Sync				_bit13
 #define AI_STOP_Edge				_bit12
-#define AI_STOP_Select(a)			((a)<<7)
+#define AI_STOP_Select(a)			(((a) & 0x1f)<<7)
 #define AI_START_Sync				_bit6
 #define AI_START_Edge				_bit5
-#define AI_START_Select(a)			(a)
+#define AI_START_Select(a)			((a) & 0x1f)
 
 #define AI_Trigger_Select_Register	63
 #define AI_START1_Polarity			_bit15
 #define AI_START2_Polarity			_bit14
 #define AI_START2_Sync				_bit13
 #define AI_START2_Edge				_bit12
-#define AI_START2_Select(a)			((a)<<7)
+#define AI_START2_Select(a)			(((a) & 0x1f) << 7)
 #define AI_START1_Sync				_bit6
 #define AI_START1_Edge				_bit5
-#define AI_START1_Select(a)			(a)
+#define AI_START1_Select(a)			((a) & 0x1f)
 
 #define AI_DIV_Load_A_Register	64
 
@@ -341,10 +351,10 @@ enum AO_FIFO_Mode_Bits
 #define AO_UI2_External_Gate_Polarity		_bit14
 #define AO_START_Polarity			_bit13
 #define AO_AOFREQ_Enable			_bit12
-#define AO_UI2_External_Gate_Select(a)		((a)<<7)
+#define AO_UI2_External_Gate_Select(a)		(((a) & 0x1f) << 7)
 #define AO_START_Sync				_bit6
 #define AO_START_Edge				_bit5
-#define AO_START_Select(a)			(a)
+#define AO_START_Select(a)			((a) & 0x1f)
 
 #define AO_Trigger_Select_Register	67
 #define AO_UI2_External_Gate_Enable		_bit15
@@ -365,8 +375,10 @@ enum AO_FIFO_Mode_Bits
 #define AO_Stop_On_BC_TC_Error			_bit3
 #define AO_Not_An_UPDATE			_bit2
 #define AO_Software_Gate			_bit1
+#define AO_Last_Gate_Disable		_bit0	/* M Series only */
 
 #define Joint_Reset_Register		72
+#define Software_Reset			_bit11
 #define AO_Configuration_End			_bit9
 #define AI_Configuration_End			_bit8
 #define AO_Configuration_Start			_bit5
@@ -420,12 +432,18 @@ enum AO_FIFO_Mode_Bits
 #define AO_Personal_Register		78
 enum AO_Personal_Bits
 {
+	AO_Interval_Buffer_Mode = 1 << 3,
 	AO_BC_Source_Select = 1 << 4,
 	AO_UPDATE_Pulse_Width = 1 << 5,
-	AO_AOFREQ_Polarity = 1 << 9,
+	AO_DMA_PIO_Control = 1 << 8,	/* M Series: reserved */
+	AO_AOFREQ_Polarity = 1 << 9,	/* M Series: reserved */
 	AO_FIFO_Enable = 1 << 10,
 	AO_TMRDACWR_Pulse_Width = 1 << 12,
+	AO_Number_Of_DAC_Packages = 1 << 14,	// 1 for "single" mode, 0 for "dual"
 };
+#define	RTSI_Trig_A_Output_Register	79
+#define	RTSI_Trig_B_Output_Register	80
+#define	RTSI_Board_Register		81
 #define Write_Strobe_0_Register		82
 #define Write_Strobe_1_Register		83
 #define Write_Strobe_2_Register		84
@@ -454,7 +472,7 @@ enum AO_Personal_Bits
 #define AI_FIFO_Mode_HF				(1<<6)
 #define AI_FIFO_Mode_NE				(0<<6)
 #define AI_External_Gate_Polarity		_bit5
-#define AI_External_Gate_Select(a)		(a)
+#define AI_External_Gate_Select(a)		((a) & 0x1f)
 
 #define G_Autoincrement_Register(a)	(68+(a))
 #define G_Command_Register(a)		(6+(a))
@@ -576,20 +594,6 @@ enum AO_Personal_Bits
 /* 16 bit registers shadowed from DAQ-STC */
 #define Window_Address			0x00
 #define Window_Data			0x02
-#if 0
-#define Interrupt_A_Ack			0x04
-#define AI_Status_1			0x04
-#define Interrupt_B_Ack			0x06
-#define AO_Status_1			0x06
-#define AI_Command_2			0x08
-#define G_Status			0x08
-#define AO_Command_2			0x0a
-#define AI_Status_2			0x0a
-#define G0_Command			0x0c
-#define AO_Status_2			0x0c
-#define G1_Command			0x0e
-#define DIO_Parallel_Input		0x0e
-#endif
 
 #define Configuration_Memory_Clear	82
 #define ADC_FIFO_Clear			83
@@ -664,6 +668,36 @@ static inline unsigned int AI_CONFIG_CHANNEL( unsigned int channel )
 #define AO_Window_Address_611x		0x18
 #define AO_Window_Data_611x		0x1e
 
+/* 6143 registers */
+#define Magic_6143			0x19 /* w8 */
+#define G0G1_DMA_Select_6143		0x0B /* w8 */
+#define PipelineDelay_6143		0x1f /* w8 */
+#define EOC_Set_6143			0x1D /* w8 */
+#define AIDMA_Select_6143		0x09 /* w8 */
+#define AIFIFO_Data_6143		0x8C /* w32 */
+#define AIFIFO_Flag_6143		0x84 /* w32 */
+#define AIFIFO_Control_6143		0x88 /* w32 */
+#define AIFIFO_Status_6143		0x88 /* w32 */
+#define AIFIFO_DMAThreshold_6143	0x90 /* w32 */
+#define AIFIFO_Words_Available_6143	0x94 /* w32 */
+
+#define Calibration_Channel_6143	0x42 /* w16 */
+#define Calibration_LowTime_6143	0x20 /* w16 */
+#define Calibration_HighTime_6143	0x22 /* w16 */
+#define Relay_Counter_Load_Val__6143	0x4C /* w32 */
+#define Signature_6143			0x50 /* w32 */
+#define Release_Date_6143		0x54 /* w32 */
+#define Release_Oldest_Date_6143	0x58 /* w32 */
+
+#define Calibration_Channel_6143_RelayOn	0x8000	/* Calibration relay switch On */
+#define Calibration_Channel_6143_RelayOff	0x4000	/* Calibration relay switch Off */
+#define Calibration_Channel_Gnd_Gnd	0x00	/* Offset Calibration */
+#define Calibration_Channel_2v5_Gnd	0x02	/* 2.5V Reference */
+#define Calibration_Channel_Pwm_Gnd	0x05	/* +/- 5V Self Cal */
+#define Calibration_Channel_2v5_Pwm	0x0a	/* PWM Calibration */
+#define Calibration_Channel_Pwm_Pwm	0x0d	/* CMRR */
+#define Calibration_Channel_Gnd_Pwm	0x0e	/* PWM Calibration */
+
 /* 671x, 611x registers */
 
 /* 671xi, 611x windowed ao registers */
@@ -757,7 +791,7 @@ enum mite_dma_channel{
 	GPC1_DMA_CHAN = 3,
 };
 
-enum{ ai_gain_16=0, ai_gain_8, ai_gain_14, ai_gain_4, ai_gain_611x };
+enum{ ai_gain_16=0, ai_gain_8, ai_gain_14, ai_gain_4, ai_gain_611x, ai_gain_622x, ai_gain_628x,  ai_gain_6143};
 enum caldac_enum { caldac_none=0, mb88341, dac8800, dac8043, ad8522,
 	ad8804, ad8842, ad8804_debug };
 enum ni_reg_type {
@@ -767,8 +801,257 @@ enum ni_reg_type {
 	ni_reg_6713 = 0x4,
 	ni_reg_67xx_mask = 0x6,
 	ni_reg_6xxx_mask = 0x7,
+	ni_reg_m_series = 0x8,
+	ni_reg_6143 = 0x10
 };
 
+static comedi_lrange range_ni_E_ao_ext;
+
+enum m_series_register_offsets
+{
+	M_Offset_CDIO_DMA_Select = 0x7,	// write
+	M_Offset_SCXI_Status = 0x7,	// read
+	M_Offset_AI_AO_Select = 0x9,	// write
+	M_Offset_SCXI_Serial_Data_In = 0x9,	// read
+	M_Offset_G0_G1_Select = 0xb,
+	M_Offset_Misc_Command = 0xf,
+	M_Offset_SCXI_Serial_Data_Out = 0x11,
+	M_Offset_SCXI_Control = 0x13,
+	M_Offset_SCXI_Output_Enable = 0x15,
+	M_Offset_AI_FIFO_Data = 0x1c,
+	M_Offset_Static_Digital_Output = 0x24,	// write
+	M_Offset_Static_Digital_Input = 0x24,	// read
+	M_Offset_DIO_Direction = 0x28,
+	M_Offset_Cal_PWM = 0x40,
+	M_Offset_AI_Config_FIFO_Data = 0x5e,
+	M_Offset_Interrupt_C_Enable = 0x88,	// write
+	M_Offset_Interrupt_C_Status = 0x88,	// read
+	M_Offset_Analog_Trigger_Control = 0x8c,
+	M_Offset_AO_Serial_Interrupt_Enable = 0xa0,
+	M_Offset_AO_Serial_Interrupt_Ack = 0xa1,	// write
+	M_Offset_AO_Serial_Interrupt_Status = 0xa1,	// read
+	M_Offset_AO_Calibration = 0xa3,
+	M_Offset_AO_FIFO_Data = 0xa4,
+	M_Offset_PFI_Filter = 0xb0,
+	M_Offset_RTSI_Filter = 0xb4,
+	M_Offset_SCXI_Legacy_Compatibility = 0xbc,
+	M_Offset_Interrupt_A_Ack = 0x104,	// write
+	M_Offset_AI_Status_1 = 0x104,	// read
+	M_Offset_Interrupt_B_Ack = 0x106,	// write
+	M_Offset_AO_Status_1 = 0x106,	// read
+	M_Offset_AI_Command_2 = 0x108,	// write
+	M_Offset_G01_Status = 0x108,	// read
+	M_Offset_AO_Command_2 = 0x10a,
+	M_Offset_AO_Status_2 = 0x10c,	// read
+	M_Offset_G0_Command = 0x10c,	// write
+	M_Offset_G1_Command = 0x10e,	// write
+	M_Offset_G0_HW_Save = 0x110,
+	M_Offset_G0_HW_Save_High = 0x110,
+	M_Offset_AI_Command_1 = 0x110,
+	M_Offset_G0_HW_Save_Low = 0x112,
+	M_Offset_AO_Command_1 = 0x112,
+	M_Offset_G1_HW_Save = 0x114,
+	M_Offset_G1_HW_Save_High = 0x114,
+	M_Offset_G1_HW_Save_Low = 0x116,
+	M_Offset_AI_Mode_1 = 0x118,
+	M_Offset_G0_Save = 0x118,
+	M_Offset_G0_Save_High = 0x118,
+	M_Offset_AI_Mode_2 = 0x11a,
+	M_Offset_G0_Save_Low = 0x11a,
+	M_Offset_AI_SI_Load_A = 0x11c,
+	M_Offset_G1_Save = 0x11c,
+	M_Offset_G1_Save_High = 0x11c,
+	M_Offset_G1_Save_Low = 0x11e,
+	M_Offset_AI_SI_Load_B = 0x120,	// write
+	M_Offset_AO_UI_Save = 0x120,	// read
+	M_Offset_AI_SC_Load_A = 0x124,	// write
+	M_Offset_AO_BC_Save = 0x124,	// read
+	M_Offset_AI_SC_Load_B = 0x128,	// write
+	M_Offset_AO_UC_Save = 0x128,	//read
+	M_Offset_AI_SI2_Load_A = 0x12c,
+	M_Offset_AI_SI2_Load_B = 0x130,
+	M_Offset_G0_Mode = 0x134,
+	M_Offset_G1_Mode = 0x136,	// write
+	M_Offset_Joint_Status_1 = 0x136,	// read
+	M_Offset_G0_Load_A = 0x138,
+	M_Offset_Joint_Status_2 = 0x13a,
+	M_Offset_G0_Load_B = 0x13c,
+	M_Offset_G1_Load_A = 0x140,
+	M_Offset_G1_Load_B = 0x144,
+	M_Offset_G0_Input_Select = 0x148,
+	M_Offset_G1_Input_Select = 0x14a,
+	M_Offset_AO_Mode_1 = 0x14c,
+	M_Offset_AO_Mode_2 = 0x14e,
+	M_Offset_AO_UI_Load_A = 0x150,
+	M_Offset_AO_UI_Load_B = 0x154,
+	M_Offset_AO_BC_Load_A = 0x158,
+	M_Offset_AO_BC_Load_B = 0x15c,
+	M_Offset_AO_UC_Load_A = 0x160,
+	M_Offset_AO_UC_Load_B = 0x164,
+	M_Offset_Clock_and_FOUT = 0x170,
+	M_Offset_IO_Bidirection_Pin = 0x172,
+	M_Offset_RTSI_Trig_Direction = 0x174,
+	M_Offset_Interrupt_Control = 0x176,
+	M_Offset_AI_Output_Control = 0x178,
+	M_Offset_Analog_Trigger_Etc = 0x17a,
+	M_Offset_AI_START_STOP_Select = 0x17c,
+	M_Offset_AI_Trigger_Select = 0x17e,
+	M_Offset_AI_SI_Save = 0x180,	// read
+	M_Offset_AI_DIV_Load_A = 0x180,	// write
+	M_Offset_AI_SC_Save = 0x184,	// read
+	M_Offset_AO_Start_Select = 0x184,	// write
+	M_Offset_AO_Trigger_Select = 0x186,
+	M_Offset_AO_Mode_3 = 0x18c,
+	M_Offset_G0_Autoincrement = 0x188,
+	M_Offset_G1_Autoincrement = 0x18a,
+	M_Offset_Joint_Reset = 0x190,
+	M_Offset_Interrupt_A_Enable = 0x192,
+	M_Offset_Interrupt_B_Enable = 0x196,
+	M_Offset_AI_Personal = 0x19a,
+	M_Offset_AO_Personal = 0x19c,
+	M_Offset_RTSI_Trig_B_Output = 0x1a0,
+	M_Offset_RTSI_Shared_MUX = 0x1a2,	
+	M_Offset_AO_Output_Control = 0x1ac,
+	M_Offset_AI_Mode_3 = 0x1ae,
+	M_Offset_Configuration_Memory_Clear = 0x1a4,
+	M_Offset_AI_FIFO_Clear = 0x1a6,
+	M_Offset_AO_FIFO_Clear = 0x1a8,
+	M_Offset_G0_Counting_Mode = 0x1b0,
+	M_Offset_G1_Counting_Mode = 0x1b2,
+	M_Offset_G0_Second_Gate = 0x1b4,
+	M_Offset_G1_Second_Gate = 0x1b6,
+	M_Offset_G0_DMA_Config = 0x1b8,	// write
+	M_Offset_G0_DMA_Status = 0x1b8,	// read
+	M_Offset_G1_DMA_Config = 0x1ba,	// write
+	M_Offset_G1_DMA_Status = 0x1ba,	// read
+	M_Offset_G0_MSeries_ABZ = 0x1c0,
+	M_Offset_G1_MSeries_ABZ = 0x1c2,
+	M_Offset_Clock_And_Fout2 = 0x1c4,
+	M_Offset_PLL_Control = 0x1c6,
+	M_Offset_PLL_Status = 0x1c8,
+	M_Offset_PFI_Output_Select_1 = 0x1d0,
+	M_Offset_PFI_Output_Select_2 = 0x1d2,
+	M_Offset_PFI_Output_Select_3 = 0x1d4,
+	M_Offset_PFI_Output_Select_4 = 0x1d6,
+	M_Offset_PFI_Output_Select_5 = 0x1d8,
+	M_Offset_PFI_Output_Select_6 = 0x1da,
+	M_Offset_PFI_DI = 0x1dc,
+	M_Offset_PFI_DO = 0x1de,
+	M_Offset_AI_Config_FIFO_Bypass = 0x218,
+	M_Offset_SCXI_DIO_Enable = 0x21c,
+	M_Offset_CDI_FIFO_Data = 0x220,	// read
+	M_Offset_CDO_FIFO_Data = 0x220,	// write
+	M_Offset_CDIO_Status = 0x224,	// read
+	M_Offset_CDIO_Command = 0x224,	// write
+	M_Offset_CDI_Mode = 0x228,
+	M_Offset_CDO_Mode = 0x22c,
+	M_Offset_CDI_Mask_Enable = 0x230,
+	M_Offset_CDO_Mask_Enable = 0x234,
+};
+static inline int M_Offset_AO_Waveform_Order(int channel)
+{
+	return 0xc2 + 0x4 * channel;
+};
+static inline int M_Offset_AO_Config_Bank(int channel)
+{
+	return 0xc3 + 0x4 * channel;
+};
+static inline int M_Offset_DAC_Direct_Data(int channel)
+{
+	return 0xc0 + 0x4 * channel;
+}
+static inline int M_Offset_Gen_PWM(int channel)
+{
+	return 0x44 + 0x2 * channel;
+}
+static inline int M_Offset_Static_AI_Control(int i)
+{
+	int offset[] =
+	{
+		0x64,
+		0x261,
+		0x262,
+		0x263,
+	};
+	if(((unsigned)i) >= sizeof(offset) / sizeof(offset[0]))
+	{
+		rt_printk("%s: invalid channel=%i\n", __FUNCTION__, i);
+		return offset[0];
+	}
+	return offset[i];
+};
+static inline int M_Offset_AO_Reference_Attenuation(int channel)
+{
+	int offset[] =
+	{
+		0x264,
+		0x265,
+		0x266,
+		0x267
+	};
+	if(((unsigned)channel) >= sizeof(offset) / sizeof(offset[0]))
+	{
+		rt_printk("%s: invalid channel=%i\n", __FUNCTION__, channel);
+		return offset[0];
+	}
+	return offset[channel];
+};
+
+enum MSeries_AI_Config_FIFO_Bypass_Bits
+{
+	MSeries_AI_Bypass_Channel_Mask = 0x7,
+	MSeries_AI_Bypass_Bank_Mask = 0x78,
+	MSeries_AI_Bypass_Cal_Sel_Pos_Mask = 0x380,
+	MSeries_AI_Bypass_Cal_Sel_Neg_Mask = 0x1c00,
+	MSeries_AI_Bypass_Mode_Mux_Mask = 0x6000,
+	MSeries_AO_Bypass_AO_Cal_Sel_Mask = 0x38000,
+	MSeries_AI_Bypass_Gain_Mask = 0x1c0000,
+	MSeries_AI_Bypass_Dither_Bit = 0x200000,
+	MSeries_AI_Bypass_Polarity_Bit = 0x400000,
+	MSeries_AI_Bypass_Config_FIFO_Bit = 0x80000000
+};
+static inline unsigned MSeries_AI_Bypass_Cal_Sel_Pos_Bits(int calibration_source)
+{
+	return (calibration_source << 7) & MSeries_AI_Bypass_Cal_Sel_Pos_Mask;
+}
+static inline unsigned MSeries_AI_Bypass_Cal_Sel_Neg_Bits(int calibration_source)
+{
+	return (calibration_source << 10) & MSeries_AI_Bypass_Cal_Sel_Pos_Mask;
+}
+static inline unsigned MSeries_AI_Bypass_Gain_Bits(int gain)
+{
+	return (gain << 18) & MSeries_AI_Bypass_Gain_Mask;
+}
+
+enum MSeries_AO_Config_Bank_Bits
+{
+	MSeries_AO_DAC_Offset_Select_Mask = 0x7,
+	MSeries_AO_DAC_Offset_0V_Bits = 0x0,
+	MSeries_AO_DAC_Offset_5V_Bits = 0x1,
+	MSeries_AO_DAC_Reference_Mask = 0x38,
+	MSeries_AO_DAC_Reference_10V_Internal_Bits = 0x0,
+	MSeries_AO_DAC_Reference_5V_Internal_Bits = 0x8,
+	MSeries_AO_Update_Timed_Bit = 0x40,
+	MSeries_AO_Bipolar_Bit = 0x80	// turns on 2's complement encoding
+};
+
+enum MSeries_AO_Reference_Attenuation_Bits
+{
+	MSeries_Attenuate_x5_Bit = 0x1
+};
+
+static inline unsigned MSeries_Cal_PWM_High_Time_Bits(unsigned count)
+{
+	return (count << 16) & 0xffff0000;
+}
+
+static inline unsigned MSeries_Cal_PWM_Low_Time_Bits(unsigned count)
+{
+	return count & 0xffff;
+}
+
+#define M_SERIES_EEPROM_SIZE 1024
+
 typedef struct ni_board_struct{
 	int device_id;
 	int isapnp_id;
@@ -786,8 +1069,8 @@ typedef struct ni_board_struct{
 	int aobits;
 
 	int ao_fifo_depth;
-	int aorangelkup;
-
+	comedi_lrange *ao_range_table;
+	
 	int reg_type;
 
 	unsigned int ao_unipolar : 1;
@@ -802,7 +1085,13 @@ static ni_board ni_boards[];
 
 #define boardtype (*(ni_board *)dev->board_ptr)
 
+#define MAX_N_AO_CHAN 8
+
 #define NI_PRIVATE_COMMON					\
+	uint16_t (*stc_readw)(comedi_device *dev, int register);	\
+	void (*stc_writew)(comedi_device *dev, uint16_t value, int register);	\
+	void (*stc_writel)(comedi_device *dev, uint32_t value, int register);	\
+	\
 	unsigned short dio_output;				\
 	unsigned short dio_control;				\
 	int ao0p,ao1p;						\
@@ -815,18 +1104,19 @@ static ni_board ni_boards[];
 	int blocksize;						\
 	int n_left;						\
 	unsigned int ai_calib_source;				\
+	unsigned int ai_calib_source_enabled;			\
 	spinlock_t window_lock; \
 								\
 	int changain_state;					\
 	unsigned int changain_spec;				\
 								\
 	unsigned int caldac_maxdata_list[MAX_N_CALDACS];	\
-	unsigned short ao[2];					\
+	unsigned short ao[MAX_N_AO_CHAN];					\
 	unsigned short caldacs[MAX_N_CALDACS];				\
 								\
 	volatile unsigned short ai_cmd2;	\
 								\
-	unsigned short ao_conf[2];				\
+	unsigned short ao_conf[MAX_N_AO_CHAN];				\
 	unsigned short ao_mode1;				\
 	unsigned short ao_mode2;				\
 	unsigned short ao_mode3;				\
@@ -841,7 +1131,7 @@ static ni_board ni_boards[];
 	int gpct_cur_operation[2];				\
 	unsigned short an_trig_etc_reg;				\
 								\
-	unsigned short ai_offset[512];				\
+	unsigned ai_offset[512];				\
 								\
 	unsigned long serial_interval_ns;                       \
 	unsigned char serial_hw_mode;                           \
@@ -850,12 +1140,17 @@ static ni_board ni_boards[];
 	volatile unsigned short int_a_enable_reg;			\
 	volatile unsigned short int_b_enable_reg;			\
 	unsigned short io_bidirection_pin_reg;			\
+	unsigned short rtsi_trig_direction_reg;			\
 								\
 	unsigned short atrig_mode;				\
 	unsigned short atrig_high;				\
 	unsigned short atrig_low;				\
-								\
+	\
+	unsigned short pwm_up_count;	\
+	unsigned short pwm_down_count;	\
+	\
 	sampl_t ai_fifo_buffer[0x2000];				\
-
+	uint8_t eeprom_buffer[M_SERIES_EEPROM_SIZE];
+	
 #endif /* _COMEDI_NI_STC_H */
 
