@@ -723,7 +723,7 @@ static int pcl812_ai_cmdtest(comedi_device *dev,comedi_subdevice *s,comedi_cmd *
 */
 static int pcl812_ai_cmd(comedi_device *dev,comedi_subdevice *s)
 {
-	unsigned int	divisor1, divisor2, i, dma_flags, bytes;
+	unsigned int	divisor1=0, divisor2=0, i, dma_flags, bytes;
 	comedi_cmd 	*cmd=&s->async->cmd;
 	
 #ifdef PCL812_EXTDEBUG
@@ -1183,11 +1183,10 @@ static int pcl812_attach(comedi_device * dev, comedi_devconfig * it)
 	printk("comedi%d: pcl812:  board=%s, ioport=0x%03x", dev->minor,
 		this_board->name, iobase);
 
-	if (check_region(iobase, this_board->io_range) < 0) {
+	if (!request_region(iobase, this_board->io_range, "pcl812")) {
 		printk("I/O port conflict\n");
 		return -EIO;
 	}
-	request_region(iobase, this_board->io_range, "pcl812");
 	dev->iobase = iobase;
 
 	if ((ret = alloc_private(dev, sizeof(pcl812_private))) < 0) {
