@@ -49,7 +49,7 @@ Devices: [Quatech] DAQP-208 (daqp), DAQP-308
 
 #include <linux/comedidev.h>
 
-#include <pcmcia/version.h>
+#include <linux/version.h>
 #include <pcmcia/cs_types.h>
 #include <pcmcia/cs.h>
 #include <pcmcia/cistpl.h>
@@ -67,7 +67,7 @@ Devices: [Quatech] DAQP-208 (daqp), DAQP-308
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
-MODULE_PARM(pc_debug, "i");
+module_param(pc_debug, int, 0644);
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
 "quatech_daqp_cs.c 1.10 2003/04/21 (Brent Baccala)";
@@ -145,22 +145,6 @@ typedef struct local_info_t {
 
 static local_info_t *dev_table[MAX_DEV] = { NULL, /* ... */ };
 
-
-/*====================================================================*/
-
-/* Parameters that can be set with 'insmod' */
-
-/* The old way: bit map of interrupts to choose from */
-/* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
-static u_int irq_mask = 0xdeb8;
-/* Newer, simpler way of listing specific interrupts */
-static int irq_list[4] = { -1 };
-
-MODULE_PARM(irq_mask, "i");
-MODULE_PARM(irq_list, "1-4i");
-
-/*====================================================================*/
-
 /* The DAQP communicates with the system through a 16 byte I/O window. */
 
 #define DAQP_FIFO_SIZE		4096
@@ -1054,12 +1038,15 @@ static int daqp_detach(comedi_device *dev)
    event is received.  The config() and release() entry points are
    used to configure or release a socket, in response to card
    insertion and ejection events.
+
+   Kernel version 2.6.16 upwards uses suspend() and resume() functions
+   instead of an event() function.
 */
 
 static void daqp_cs_config(dev_link_t *link);
 static void daqp_cs_release(u_long arg);
-static int daqp_cs_event(event_t event, int priority,
-		       event_callback_args_t *args);
+static int daqp_cs_suspend(struct pcmcia_device *p_dev);
+static int daqp_cs_resume(struct pcmcia_device *p_dev);
 
 /*
    The attach() and detach() entry points are used to create and destroy
@@ -1067,8 +1054,8 @@ static int daqp_cs_event(event_t event, int priority,
    needed to manage one actual PCMCIA card.
 */
 
-static dev_link_t *daqp_cs_attach(void);
-static void daqp_cs_detach(dev_link_t *);
+static  int daqp_cs_attach(struct pcmcia_device *);
+static void daqp_cs_detach(struct pcmcia_device *);
 
 /*
    The dev_info variable is the "key" that is used to match up this
@@ -1091,12 +1078,11 @@ static dev_info_t dev_info = "quatech_daqp_cs";
     
 ======================================================================*/
 
-static dev_link_t *daqp_cs_attach(void)
+static  int daqp_cs_attach(struct pcmcia_device *p_dev)
 {
     local_info_t *local;
     dev_link_t *link;
-    client_reg_t client_reg;
-    int ret, i;
+    int i;
     
     DEBUG(0, "daqp_cs_attach()\n");
 
@@ -1104,32 +1090,22 @@ static dev_link_t *daqp_cs_attach(void)
       if (dev_table[i] == NULL) break;
     if (i == MAX_DEV) {
       printk(KERN_NOTICE "daqp_cs: no devices available\n");
-      return NULL;
+      return -ENODEV;
     }
     
     /* Allocate space for private device-specific data */
     local = kmalloc(sizeof(local_info_t), GFP_KERNEL);
-    if (!local) return NULL;
+    if (!local) return -ENOMEM;
     memset(local, 0, sizeof(local_info_t));
 
     local->table_index = i;
     dev_table[i] = local;
     link = &local->link;
     link->priv = local;
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-    /* Initialize the dev_link_t structure */
-    link->release.function = &daqp_cs_release;
-    link->release.data = (u_long)link;
-#endif
-    /* Interrupt setup */
+    
+	/* Interrupt setup */
     link->irq.Attributes = IRQ_TYPE_EXCLUSIVE | IRQ_HANDLE_PRESENT;
-    link->irq.IRQInfo1 = IRQ_INFO2_VALID|IRQ_LEVEL_ID;
-    if (irq_list[0] == -1)
-	link->irq.IRQInfo2 = irq_mask;
-    else
-	for (i = 0; i < 4; i++)
-	    link->irq.IRQInfo2 |= 1 << irq_list[i];
+    link->irq.IRQInfo1 = IRQ_LEVEL_ID;
     link->irq.Handler = daqp_interrupt;
     link->irq.Instance = local;
     
@@ -1144,24 +1120,12 @@ static dev_link_t *daqp_cs_attach(void)
     link->conf.Vcc = 50;
     link->conf.IntType = INT_MEMORY_AND_IO;
 
-    /* Register with Card Services */
-    client_reg.dev_info = &dev_info;
-    client_reg.Attributes = INFO_IO_CLIENT | INFO_CARD_SHARE;
-    client_reg.EventMask =
-	CS_EVENT_CARD_INSERTION | CS_EVENT_CARD_REMOVAL |
-	CS_EVENT_RESET_PHYSICAL | CS_EVENT_CARD_RESET |
-	CS_EVENT_PM_SUSPEND | CS_EVENT_PM_RESUME;
-    client_reg.event_handler = &daqp_cs_event;
-    client_reg.Version = 0x0210;
-    client_reg.event_callback_args.client_data = link;
-    ret = pcmcia_register_client(&link->handle, &client_reg);
-    if (ret != CS_SUCCESS) {
-		cs_error(link->handle, RegisterClient, ret);
-		daqp_cs_detach(link);
-		return NULL;
-    }
+    link->handle = p_dev;
+    p_dev->instance = link;
+    link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
+    daqp_cs_config(link);
 
-    return link;
+    return 0;
 } /* daqp_cs_attach */
 
 /*======================================================================
@@ -1173,8 +1137,9 @@ static dev_link_t *daqp_cs_attach(void)
 
 ======================================================================*/
 
-static void daqp_cs_detach(dev_link_t *link)
+static void daqp_cs_detach(struct pcmcia_device *p_dev)
 {
+    dev_link_t *link = dev_to_instance(p_dev);
     local_info_t *dev = link->priv;
 
     DEBUG(0, "daqp_cs_detach(0x%p)\n", link);
@@ -1186,18 +1151,10 @@ static void daqp_cs_detach(dev_link_t *link)
        detach().
     */
     if (link->state & DEV_CONFIG) {
-#ifdef PCMCIA_DEBUG
-	printk(KERN_DEBUG "daqp_cs: detach postponed, '%s' "
-	       "still locked\n", link->dev->dev_name);
-#endif
-	link->state |= DEV_STALE_LINK;
-	return;
+	dev->stop = 1;
+	daqp_cs_release((u_long)link);
     }
 
-    /* Break the link with Card Services */
-    if (link->handle)
-		pcmcia_deregister_client(link->handle);
-    
     /* Unlink device structure, and free it */
     dev_table[dev->table_index] = NULL;
     kfree(dev);
@@ -1417,18 +1374,6 @@ static void daqp_cs_release(u_long arg)
 
     DEBUG(0, "daqp_cs_release(0x%p)\n", link);
 
-    /*
-       If the device is currently in use, we won't release until it
-       is actually closed, because until then, we can't be sure that
-       no one will try to access the device or its data structures.
-    */
-    if (link->open) {
-	DEBUG(1, "daqp_cs: release postponed, '%s' still open\n",
-	      link->dev->dev_name);
-	link->state |= DEV_STALE_CONFIG;
-	return;
-    }
-
     /* Unlink the device chain */
     link->dev = NULL;
 
@@ -1446,9 +1391,6 @@ static void daqp_cs_release(u_long arg)
 		pcmcia_release_irq(link->handle, &link->irq);
     link->state &= ~DEV_CONFIG;
     
-    if (link->state & DEV_STALE_LINK)
-	daqp_cs_detach(link);
-    
 } /* daqp_cs_release */
 
 /*======================================================================
@@ -1463,82 +1405,66 @@ static void daqp_cs_release(u_long arg)
     
 ======================================================================*/
 
-static int daqp_cs_event(event_t event, int priority,
-		       event_callback_args_t *args)
+static int daqp_cs_suspend(struct pcmcia_device *p_dev)
 {
-    dev_link_t *link = args->client_data;
-    local_info_t *dev = link->priv;
-    
-    DEBUG(1, "daqp_cs_event(0x%06x)\n", event);
-    
-    switch (event) {
-    case CS_EVENT_CARD_REMOVAL:
-	link->state &= ~DEV_PRESENT;
-	if (link->state & DEV_CONFIG) {
-	    dev->stop = 1;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-	    link->release.expires = jiffies + HZ/20;
-	    add_timer(&link->release);
-#else
-		daqp_cs_release((ulong)link);
-#endif
-	}
-	break;
-    case CS_EVENT_CARD_INSERTION:
-	link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
-	daqp_cs_config(link);
-	break;
-    case CS_EVENT_PM_SUSPEND:
-	link->state |= DEV_SUSPEND;
-	/* Fall through... */
-    case CS_EVENT_RESET_PHYSICAL:
-	/* Mark the device as stopped, to block IO until later */
-        dev->stop = 1;
-	if (link->state & DEV_CONFIG)
-	    pcmcia_release_configuration(link->handle);
-	break;
-    case CS_EVENT_PM_RESUME:
-	link->state &= ~DEV_SUSPEND;
-	/* Fall through... */
-    case CS_EVENT_CARD_RESET:
-	if (link->state & DEV_CONFIG)
-	    pcmcia_request_configuration(link->handle, &link->conf);
-	dev->stop = 0;
-	break;
-    }
+    dev_link_t *link = dev_to_instance(p_dev);
+    local_info_t *local = link->priv;
+
+    link->state |= DEV_SUSPEND;
+    /* Mark the device as stopped, to block IO until later */
+    local->stop = 1;
+    if (link->state & DEV_CONFIG)
+	pcmcia_release_configuration(link->handle);
+
     return 0;
-} /* daqp_cs_event */
+}
+
+static int daqp_cs_resume(struct pcmcia_device *p_dev)
+{
+    dev_link_t *link = dev_to_instance(p_dev);
+    local_info_t *local = link->priv;
+
+    link->state &= ~DEV_SUSPEND;
+    if (link->state & DEV_CONFIG)
+	pcmcia_request_configuration(link->handle, &link->conf);
+    local->stop = 0;
+
+    return 0;
+}
 
 /*====================================================================*/
 
 #ifdef MODULE
 
+static struct pcmcia_device_id daqp_cs_id_table[] =
+{
+	PCMCIA_DEVICE_MANF_CARD(0x0137, 0x0027),
+	PCMCIA_DEVICE_NULL
+};
+MODULE_DEVICE_TABLE(pcmcia, daqp_cs_id_table);
+
 struct pcmcia_driver daqp_cs_driver =
 {
-	.attach = daqp_cs_attach,
-	.detach = daqp_cs_detach,
+	.probe = daqp_cs_attach,
+	.remove = daqp_cs_detach,
+	.suspend = daqp_cs_suspend,
+	.resume = daqp_cs_resume,
+	.id_table = daqp_cs_id_table,
 	.owner = THIS_MODULE,
 	.drv = {
-		.name = "quatech_daqp_cs",
+		.name = dev_info,
 	},	
 };
 
-int init_module(void)
+int __init init_module(void)
 {
-    servinfo_t serv;
     DEBUG(0, "%s\n", version);
-    pcmcia_get_card_services_info(&serv);
-    if (serv.Revision != CS_RELEASE_CODE) {
-	printk(KERN_NOTICE "daqp_cs: Card Services release "
-	       "does not match!\n");
-	return -1;
-    }
 	pcmcia_register_driver(&daqp_cs_driver);
     comedi_driver_register(&driver_daqp);
     return 0;
 }
 
-void cleanup_module(void)
+void __exit cleanup_module(void)
 {
     int i;
 
@@ -1554,8 +1480,9 @@ void cleanup_module(void)
 	 * a detach, which freed the memory, so make sure we don't
 	 * do it again or oops()... that's why this test is here
 	 */
-	if (dev_table[i])
-	  daqp_cs_detach(&dev_table[i]->link);
+	if (dev_table[i]) {
+	  daqp_cs_detach(dev_table[i]->link.handle);
+	}
       }
     }
 }
