@@ -347,6 +347,7 @@ typedef struct{
 
 					/* PCI device info */
     struct pci_dev *pci_dev;
+    int got_regions;			/* non-zero if PCI regions owned */
 
     /* channel list info */
     /* chanBipolar tracks whether a channel is bipolar (and needs +2048) */
@@ -774,43 +775,42 @@ static int rtd_attach (
     /*
      * Probe the device to determine what device in the series it is.
      */
-	for(pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ; 
+	for(pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ;
 		pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pcidev)) {
-	if (pcidev->vendor == PCI_VENDOR_ID_RTD) {
-	    if (it->options[0] || it->options[1]) {
-		if (pcidev->bus->number == it->options[0]
-		    && PCI_SLOT(pcidev->devfn) == it->options[1]) {
-		    DPRINTK("rtd520: found bus=%d slot=%d\n",
-			   it->options[0], it->options[1]);
-		    break;		/* found it */
+		if (pcidev->vendor == PCI_VENDOR_ID_RTD) {
+			if (it->options[0] || it->options[1]) {
+				if (pcidev->bus->number != it->options[0]
+					|| PCI_SLOT(pcidev->devfn) != it->options[1]) {
+					continue;
+				}
+			}
+			break;		/* found one */
 		}
-	    } else {		/* specific board/slot not specified */
-		break;		/* found one */
-	    }
 	}
-    }
-
     if (!pcidev) {
-	if (it->options[0] && it->options[1]) {
-	    printk ("No RTD card at bus=%d slot=%d.\n",
-		    it->options[0], it->options[1]);
-	} else {
-	    printk ("No RTD card found.\n");
+		if (it->options[0] && it->options[1]) {
+			printk ("No RTD card at bus=%d slot=%d.\n",
+				it->options[0], it->options[1]);
+		} else {
+			printk ("No RTD card found.\n");
+		}
+		return -EIO;
+	}
+	devpriv->pci_dev = pcidev;
+	if (pcidev->device != thisboard->device_id) {
+		printk ("Found an RTD card, but not the supported type (%x).\n",
+			pcidev->device);
+		return -EIO;
 	}
-	return -EIO;
-    }
-
-    devpriv->pci_dev = pcidev;
-    if (pcidev->device != thisboard->device_id) {
-	printk ("Found an RTD card, but not the supported type (%x).\n",
-		pcidev->device);
-	return -EIO;
-    }
     dev->board_name = thisboard->name;
 
-    if((ret=pci_enable_device(pcidev))<0){
-	return ret;
-    }
+	if((ret=pci_enable_device(pcidev))<0){
+		return ret;
+	}
+	if((ret=pci_request_regions(pcidev, "rtd520"))<0){
+		return ret;
+	}
+	devpriv->got_regions = 1;
 
     /*
      * Initialize base addresses
@@ -825,6 +825,10 @@ static int rtd_attach (
     devpriv->las1 = ioremap_nocache(physLas1, LAS1_PCISIZE);
     devpriv->lcfg = ioremap_nocache(physLcfg, LCFG_PCISIZE);
 
+    if(!devpriv->las0 || !devpriv->las1 || !devpriv->lcfg){
+    	return -ENOMEM;
+    }
+
     DPRINTK ("%s: LAS0=%lx, LAS1=%lx, CFG=%lx.\n", dev->board_name,
 	     physLas0, physLas1, physLcfg);
     {					/* The RTD driver does this */
@@ -917,18 +921,14 @@ static int rtd_attach (
     s->n_chan=3;
     s->maxdata=0xffff;
 
-    /* check if our interrupt is available and get it */
-    dev->irq = devpriv->pci_dev->irq;
-    if (dev->irq > 0) {
-	if((ret=comedi_request_irq (dev->irq, rtd_interrupt,
-				    SA_SHIRQ, "rtd520", dev)) < 0) {
-	    printk("Could not get interrupt! (%d)\n", dev->irq);
-	    return ret;
+	/* check if our interrupt is available and get it */
+	if((ret=comedi_request_irq (devpriv->pci_dev->irq, rtd_interrupt,
+		SA_SHIRQ, "rtd520", dev)) < 0) {
+		printk("Could not get interrupt! (%d)\n", devpriv->pci_dev->irq);
+		return ret;
 	}
+	dev->irq = devpriv->pci_dev->irq;
 	printk("( irq=%d )", dev->irq);
-    } else {
-	printk("( NO IRQ )");
-    }
 
 #ifdef USE_DMA
     if (dev->irq > 0) {
@@ -1086,23 +1086,30 @@ static int rtd_detach (
     int index;
 #endif
 
-    DPRINTK("comedi%d: rtd520: removing (%ld ints)\n(int status 0x%x, overrun status 0x%x, fifo status 0x%x)...\n",
-	   dev->minor, devpriv->intCount, 
-	   0xffff & RtdInterruptStatus (dev),
-	   0xffff & RtdInterruptOverrunStatus (dev),
-	   (0xffff & RtdFifoStatus (dev)) ^ 0x6666);
+    DPRINTK("comedi%d: rtd520: removing (%ld ints)\n",
+	   dev->minor, (devpriv ? devpriv->intCount : 0L));
+    if (devpriv && devpriv->lcfg) {
+	DPRINTK("(int status 0x%x, overrun status 0x%x, fifo status 0x%x)...\n",
+	       0xffff & RtdInterruptStatus (dev),
+	       0xffff & RtdInterruptOverrunStatus (dev),
+	       (0xffff & RtdFifoStatus (dev)) ^ 0x6666);
+    }
 
     if (devpriv) {
 	/* Shut down any board ops by resetting it */
 #ifdef USE_DMA
-	RtdDma0Control (dev, 0);	/* disable DMA */
-	RtdDma1Control (dev, 0);	/* disable DMA */
-	RtdPlxInterruptWrite (dev, ICS_PIE | ICS_PLIE);
+	if (devpriv->lcfg) {
+	    RtdDma0Control (dev, 0);	/* disable DMA */
+	    RtdDma1Control (dev, 0);	/* disable DMA */
+	    RtdPlxInterruptWrite (dev, ICS_PIE | ICS_PLIE);
+	}
 #endif /* USE_DMA */
-	RtdResetBoard (dev);
-	RtdInterruptMask (dev, 0);
-	RtdInterruptClearMask (dev,~0);
-	RtdInterruptClear(dev);		/* clears bits set by mask */
+	if (devpriv->las0) {
+	    RtdResetBoard (dev);
+	    RtdInterruptMask (dev, 0);
+	    RtdInterruptClearMask (dev,~0);
+	    RtdInterruptClear(dev);		/* clears bits set by mask */
+	}
 
 #ifdef USE_DMA
 	/* release DMA */
@@ -1145,7 +1152,12 @@ static int rtd_detach (
 	    iounmap (devpriv->lcfg);
 	}
 	if (devpriv->pci_dev) {
-	    pci_dev_put(devpriv->pci_dev);
+		if(devpriv->got_regions)
+		{
+			pci_release_regions(devpriv->pci_dev);
+			pci_disable_device(devpriv->pci_dev);
+		}    
+		pci_dev_put(devpriv->pci_dev);
 	}
     }
 
