@@ -2,9 +2,6 @@
 #define __MULTI_CONFIG_H_
 
 #include <config.h>
-#ifdef CONFIG_COMEDI_RTAI
-//#include <modbuild/config.h>
-#endif
 #include_next <linux/config.h>
 
 #endif
